@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2009 Red Hat, Inc.
+ * Copyright (c) 2006-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -32,6 +32,13 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+/**
+ * @file
+ *
+ * @warning DO NOT USE SYMBOLS IN THIS FILE
+ */
+
 #ifndef MAIN_H_DEFINED
 #define MAIN_H_DEFINED
 
@@ -39,19 +46,12 @@
 #define FALSE 0
 #include <corosync/corotypes.h>
 #include <corosync/hdb.h>
-#include <corosync/totem/coropoll.h>
+#include <qb/qbloop.h>
 #include <corosync/totem/totempg.h>
-#include <corosync/engine/objdb.h>
-#include <corosync/engine/config.h>
-#include <corosync/engine/coroapi.h>
+#include <corosync/coroapi.h>
 
-/*
- * DO NOT USE SYMBOLS IN THIS FILE
- */
 extern unsigned long long *(*main_clm_get_by_nodeid) (unsigned int node_id);
 
-extern void main_get_config_modules(struct config_iface_ver0 ***modules, int *num);
-
 extern int main_mcast (
 	const struct iovec *iovec,
 	unsigned int iov_len,
@@ -65,6 +65,62 @@ extern void corosync_shutdown_request (void);
 
 extern void corosync_state_dump (void);
 
-extern hdb_handle_t corosync_poll_handle_get (void);
+extern qb_loop_t *cs_poll_handle_get (void);
+
+extern int cs_poll_dispatch_add (qb_loop_t * handle,
+		int fd,
+		int events,
+		void *data,
+
+		int (*dispatch_fn) (int fd,
+			int revents,
+			void *data));
+
+extern int cs_poll_dispatch_delete (
+		qb_loop_t * handle,
+		int fd);
+
+
+extern int corosync_sending_allowed (
+	unsigned int service,
+	unsigned int id,
+	const void *msg,
+	void *sending_allowed_private_data);
+
+extern void corosync_sending_allowed_release (void *sending_allowed_private_data);
+
+extern void corosync_recheck_the_q_level(void *data);
+
+extern void cs_ipcs_init(void);
+
+extern const char *cs_ipcs_service_init(struct corosync_service_engine *service);
+
+extern void cs_ipcs_stats_update(void);
+
+extern int32_t cs_ipcs_service_destroy(int32_t service_id);
+
+extern int32_t cs_ipcs_q_level_get(void);
+
+extern int cs_ipcs_dispatch_send(void *conn, const void *msg, size_t mlen);
+extern int cs_ipcs_dispatch_iov_send (void *conn,
+	const struct iovec *iov,
+	unsigned int iov_len);
+
+extern int cs_ipcs_response_send(void *conn, const void *msg, size_t mlen);
+extern int cs_ipcs_response_iov_send (void *conn,
+	const struct iovec *iov,
+	unsigned int iov_len);
+
+extern void cs_ipcs_sync_state_changed(int32_t sync_in_process);
+
+extern void *cs_ipcs_private_data_get(void *conn);
+
+extern void cs_ipc_refcnt_inc(void *conn);
+
+extern void cs_ipc_refcnt_dec(void *conn);
+
+extern void cs_ipc_allow_connections(int32_t allow);
+
+int coroparse_configparse (const char **error_string);
 
 #endif /* MAIN_H_DEFINED */
