@@ -1,10 +1,11 @@
 /*
  * Copyright (c) 2002-2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2010 Red Hat, Inc.
+ * Copyright (c) 2006-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
+ *         Jan Friesse (jfriesse@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -50,26 +51,17 @@
 
 #include <corosync/swab.h>
 #include <corosync/list.h>
+#include <qb/qbdefs.h>
 #include <corosync/totem/totem.h>
-#include <corosync/engine/objdb.h>
-#include <corosync/engine/config.h>
-#include <corosync/engine/logsys.h>
-
-#ifdef HAVE_LIBNSS
-#include <nss.h>
-#include <pk11pub.h>
-#include <pkcs11.h>
-#include <prerror.h>
-#endif
+#include <corosync/config.h>
+#include <corosync/logsys.h>
+#include <corosync/icmap.h>
 
 #include "util.h"
 #include "totemconfig.h"
-#include "tlist.h" /* for HZ */
 
 #define TOKEN_RETRANSMITS_BEFORE_LOSS_CONST	4
 #define TOKEN_TIMEOUT				1000
-#define TOKEN_RETRANSMIT_TIMEOUT		(int)(TOKEN_TIMEOUT / (TOKEN_RETRANSMITS_BEFORE_LOSS_CONST + 0.2))
-#define TOKEN_HOLD_TIMEOUT			(int)(TOKEN_RETRANSMIT_TIMEOUT * 0.8 - (1000/(int)HZ))
 #define JOIN_TIMEOUT				50
 #define MERGE_TIMEOUT				200
 #define DOWNCHECK_TIMEOUT			1000
@@ -85,212 +77,375 @@
 #define RRP_PROBLEM_COUNT_THRESHOLD_MIN		2
 #define RRP_AUTORECOVERY_CHECK_TIMEOUT		1000
 
+#define DEFAULT_PORT				5405
+
 static char error_string_response[512];
-static struct objdb_iface_ver0 *global_objdb;
 
-static void add_totem_config_notification(
-	struct objdb_iface_ver0 *objdb,
-	struct totem_config *totem_config,
-	hdb_handle_t totem_object_handle);
+static void add_totem_config_notification(struct totem_config *totem_config);
 
-
-/* These just makes the code below a little neater */
-static inline int objdb_get_string (
-	const struct objdb_iface_ver0 *objdb,
-	hdb_handle_t object_service_handle,
-	const char *key, const char **value)
+static void totem_volatile_config_read (struct totem_config *totem_config)
 {
-	int res;
+	char *str;
 
-	*value = NULL;
-	if ( !(res = objdb->object_key_get (object_service_handle,
-		key,
-		strlen (key),
-		(void *)value,
-		NULL))) {
-
-		if (*value) {
-			return 0;
-		}
-	}
-	return -1;
-}
-
-static inline void objdb_get_int (
-	const struct objdb_iface_ver0 *objdb,
-	hdb_handle_t object_service_handle,
-	const char *key, unsigned int *intvalue)
-{
-	char *value = NULL;
-
-	if (!objdb->object_key_get (object_service_handle,
-		key,
-		strlen (key),
-		(void *)&value,
-		NULL)) {
-
-		if (value) {
-			*intvalue = atoi(value);
-		}
+	icmap_get_uint32("totem.token", &totem_config->token_timeout);
+	icmap_get_uint32("totem.token_retransmit", &totem_config->token_retransmit_timeout);
+	icmap_get_uint32("totem.hold", &totem_config->token_hold_timeout);
+	icmap_get_uint32("totem.token_retransmits_before_loss_const", &totem_config->token_retransmits_before_loss_const);
+	icmap_get_uint32("totem.join", &totem_config->join_timeout);
+	icmap_get_uint32("totem.send_join", &totem_config->send_join_timeout);
+	icmap_get_uint32("totem.consensus", &totem_config->consensus_timeout);
+	icmap_get_uint32("totem.merge", &totem_config->merge_timeout);
+	icmap_get_uint32("totem.downcheck", &totem_config->downcheck_timeout);
+	icmap_get_uint32("totem.fail_recv_const", &totem_config->fail_to_recv_const);
+	icmap_get_uint32("totem.seqno_unchanged_const", &totem_config->seqno_unchanged_const);
+	icmap_get_uint32("totem.rrp_token_expired_timeout", &totem_config->rrp_token_expired_timeout);
+	icmap_get_uint32("totem.rrp_problem_count_timeout", &totem_config->rrp_problem_count_timeout);
+	icmap_get_uint32("totem.rrp_problem_count_threshold", &totem_config->rrp_problem_count_threshold);
+	icmap_get_uint32("totem.rrp_problem_count_mcast_threshold", &totem_config->rrp_problem_count_mcast_threshold);
+	icmap_get_uint32("totem.rrp_autorecovery_check_timeout", &totem_config->rrp_autorecovery_check_timeout);
+	icmap_get_uint32("totem.heartbeat_failures_allowed", &totem_config->heartbeat_failures_allowed);
+	icmap_get_uint32("totem.max_network_delay", &totem_config->max_network_delay);
+	icmap_get_uint32("totem.window_size", &totem_config->window_size);
+	icmap_get_uint32("totem.max_messages", &totem_config->max_messages);
+	icmap_get_uint32("totem.miss_count_const", &totem_config->miss_count_const);
+	if (icmap_get_string("totem.vsftype", &str) == CS_OK) {
+		totem_config->vsf_type = str;
 	}
 }
 
-static unsigned int totem_handle_find (
-	struct objdb_iface_ver0 *objdb,
-	hdb_handle_t *totem_find_handle)  {
 
-	hdb_handle_t object_find_handle;
-	unsigned int res;
+static int totem_get_crypto(struct totem_config *totem_config)
+{
+	char *str;
+	const char *tmp_cipher;
+	const char *tmp_hash;
 
-	/*
-	 * Find a network section
-	 */
-	objdb->object_find_create (
-		OBJECT_PARENT_HANDLE,
-		"network",
-		strlen ("network"),
-		&object_find_handle);
+	tmp_hash = "sha1";
+	tmp_cipher = "aes256";
 
-	res = objdb->object_find_next (
-		object_find_handle,
-		totem_find_handle);
-
-	objdb->object_find_destroy (object_find_handle);
-
-	/*
-	 * Network section not found in configuration, checking for totem
-	 */
-	if (res == -1) {
-		objdb->object_find_create (
-			OBJECT_PARENT_HANDLE,
-			"totem",
-			strlen ("totem"),
-			&object_find_handle);
-
-		res = objdb->object_find_next (
-			object_find_handle,
-			totem_find_handle);
-
-		objdb->object_find_destroy (object_find_handle);
+	if (icmap_get_string("totem.secauth", &str) == CS_OK) {
+		if (strcmp (str, "off") == 0) {
+			tmp_hash = "none";
+			tmp_cipher = "none";
+		}
+		free(str);
 	}
 
+	if (icmap_get_string("totem.crypto_cipher", &str) == CS_OK) {
+		if (strcmp(str, "none") == 0) {
+			tmp_cipher = "none";
+		}
+		if (strcmp(str, "aes256") == 0) {
+			tmp_cipher = "aes256";
+		}
+		if (strcmp(str, "aes192") == 0) {
+			tmp_cipher = "aes192";
+		}
+		if (strcmp(str, "aes128") == 0) {
+			tmp_cipher = "aes128";
+		}
+		if (strcmp(str, "3des") == 0) {
+			tmp_cipher = "3des";
+		}
+		free(str);
+	}
+
+	if (icmap_get_string("totem.crypto_hash", &str) == CS_OK) {
+		if (strcmp(str, "none") == 0) {
+			tmp_hash = "none";
+		}
+		if (strcmp(str, "md5") == 0) {
+			tmp_hash = "md5";
+		}
+		if (strcmp(str, "sha1") == 0) {
+			tmp_hash = "sha1";
+		}
+		if (strcmp(str, "sha256") == 0) {
+			tmp_hash = "sha256";
+		}
+		if (strcmp(str, "sha384") == 0) {
+			tmp_hash = "sha384";
+		}
+		if (strcmp(str, "sha512") == 0) {
+			tmp_hash = "sha512";
+		}
+		free(str);
+	}
+
+	if ((strcmp(tmp_cipher, "none") != 0) &&
+	    (strcmp(tmp_hash, "none") == 0)) {
+		return -1;
+	}
+
+	free(totem_config->crypto_cipher_type);
+	free(totem_config->crypto_hash_type);
+
+	totem_config->crypto_cipher_type = strdup(tmp_cipher);
+	totem_config->crypto_hash_type = strdup(tmp_hash);
+
+	return 0;
+}
+
+static uint16_t generate_cluster_id (const char *cluster_name)
+{
+	int i;
+	int value = 0;
+
+	for (i = 0; i < strlen(cluster_name); i++) {
+		value <<= 1;
+		value += cluster_name[i];
+	}
+
+	return (value & 0xFFFF);
+}
+
+static int get_cluster_mcast_addr (
+		const char *cluster_name,
+		const struct totem_ip_address *bindnet,
+		unsigned int ringnumber,
+		int ip_version,
+		struct totem_ip_address *res)
+{
+	uint16_t clusterid;
+	char addr[INET6_ADDRSTRLEN + 1];
+	int err;
+
+	if (cluster_name == NULL) {
+		return (-1);
+	}
+
+	clusterid = generate_cluster_id(cluster_name) + ringnumber;
+	memset (res, 0, sizeof(res));
+
+	switch (bindnet->family) {
+	case AF_INET:
+		snprintf(addr, sizeof(addr), "239.192.%d.%d", clusterid >> 8, clusterid % 0xFF);
+		break;
+	case AF_INET6:
+		snprintf(addr, sizeof(addr), "ff15::%x", clusterid);
+		break;
+	default:
+		/*
+		 * Unknown family
+		 */
+		return (-1);
+	}
+
+	err = totemip_parse (res, addr, ip_version);
+
+	return (err);
+}
+
+static int find_local_node_in_nodelist(struct totem_config *totem_config)
+{
+	icmap_iter_t iter;
+	const char *iter_key;
+	int res = 0;
+	int node_pos;
+	int local_node_pos = -1;
+	struct totem_ip_address bind_addr;
+	int interface_up, interface_num;
+	char tmp_key[ICMAP_KEYNAME_MAXLEN];
+	char *node_addr_str;
+	struct totem_ip_address node_addr;
+
+	res = totemip_iface_check(&totem_config->interfaces[0].bindnet,
+		&bind_addr, &interface_up, &interface_num,
+		totem_config->clear_node_high_bit);
 	if (res == -1) {
 		return (-1);
 	}
 
-	return (0);
+	iter = icmap_iter_init("nodelist.node.");
+	while ((iter_key = icmap_iter_next(iter, NULL, NULL)) != NULL) {
+		res = sscanf(iter_key, "nodelist.node.%u.%s", &node_pos, tmp_key);
+		if (res != 2) {
+			continue;
+		}
+
+		if (strcmp(tmp_key, "ring0_addr") != 0) {
+			continue;
+		}
+
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.ring0_addr", node_pos);
+		if (icmap_get_string(tmp_key, &node_addr_str) != CS_OK) {
+			continue;
+		}
+
+		res = totemip_parse (&node_addr, node_addr_str, totem_config->ip_version);
+		free(node_addr_str);
+		if (res == -1) {
+			continue ;
+		}
+
+		if (totemip_equal(&bind_addr, &node_addr)) {
+			local_node_pos = node_pos;
+		}
+	}
+	icmap_iter_finalize(iter);
+
+	return (local_node_pos);
 }
 
-static void totem_volatile_config_read (
-	struct objdb_iface_ver0 *objdb,
-	struct totem_config *totem_config,
-	hdb_handle_t object_totem_handle)
+static void put_nodelist_members_to_config(struct totem_config *totem_config)
 {
-	objdb_get_int (objdb,object_totem_handle, "token", &totem_config->token_timeout);
+	icmap_iter_t iter, iter2;
+	const char *iter_key, *iter_key2;
+	int res = 0;
+	int node_pos;
+	char tmp_key[ICMAP_KEYNAME_MAXLEN];
+	char tmp_key2[ICMAP_KEYNAME_MAXLEN];
+	char *node_addr_str;
+	int member_count;
+	unsigned int ringnumber = 0;
 
-	objdb_get_int (objdb,object_totem_handle, "token_retransmit", &totem_config->token_retransmit_timeout);
+	iter = icmap_iter_init("nodelist.node.");
+	while ((iter_key = icmap_iter_next(iter, NULL, NULL)) != NULL) {
+		res = sscanf(iter_key, "nodelist.node.%u.%s", &node_pos, tmp_key);
+		if (res != 2) {
+			continue;
+		}
 
-	objdb_get_int (objdb,object_totem_handle, "hold", &totem_config->token_hold_timeout);
+		if (strcmp(tmp_key, "ring0_addr") != 0) {
+			continue;
+		}
 
-	objdb_get_int (objdb,object_totem_handle, "token_retransmits_before_loss_const", &totem_config->token_retransmits_before_loss_const);
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.", node_pos);
+		iter2 = icmap_iter_init(tmp_key);
+		while ((iter_key2 = icmap_iter_next(iter2, NULL, NULL)) != NULL) {
+			res = sscanf(iter_key2, "nodelist.node.%u.ring%u%s", &node_pos, &ringnumber, tmp_key2);
+			if (res != 3 || strcmp(tmp_key2, "_addr") != 0) {
+				continue;
+			}
 
-	objdb_get_int (objdb,object_totem_handle, "join", &totem_config->join_timeout);
-	objdb_get_int (objdb,object_totem_handle, "send_join", &totem_config->send_join_timeout);
+			if (icmap_get_string(iter_key2, &node_addr_str) != CS_OK) {
+				continue;
+			}
 
-	objdb_get_int (objdb,object_totem_handle, "consensus", &totem_config->consensus_timeout);
+			member_count = totem_config->interfaces[ringnumber].member_count;
 
-	objdb_get_int (objdb,object_totem_handle, "merge", &totem_config->merge_timeout);
+			res = totemip_parse(&totem_config->interfaces[ringnumber].member_list[member_count],
+						node_addr_str, totem_config->ip_version);
+			if (res != -1) {
+				totem_config->interfaces[ringnumber].member_count++;
+			}
+			free(node_addr_str);
+		}
 
-	objdb_get_int (objdb,object_totem_handle, "downcheck", &totem_config->downcheck_timeout);
+		icmap_iter_finalize(iter2);
+	}
 
-	objdb_get_int (objdb,object_totem_handle, "fail_recv_const", &totem_config->fail_to_recv_const);
-
-	objdb_get_int (objdb,object_totem_handle, "seqno_unchanged_const", &totem_config->seqno_unchanged_const);
-
-	objdb_get_int (objdb,object_totem_handle, "rrp_token_expired_timeout", &totem_config->rrp_token_expired_timeout);
-
-	objdb_get_int (objdb,object_totem_handle, "rrp_problem_count_timeout", &totem_config->rrp_problem_count_timeout);
-
-	objdb_get_int (objdb,object_totem_handle, "rrp_problem_count_threshold", &totem_config->rrp_problem_count_threshold);
-
-	objdb_get_int (objdb,object_totem_handle, "rrp_problem_count_mcast_threshold", &totem_config->rrp_problem_count_mcast_threshold);
-
-	objdb_get_int (objdb,object_totem_handle, "rrp_autorecovery_check_timeout", &totem_config->rrp_autorecovery_check_timeout);
-
-	objdb_get_int (objdb,object_totem_handle, "heartbeat_failures_allowed", &totem_config->heartbeat_failures_allowed);
-
-	objdb_get_int (objdb,object_totem_handle, "max_network_delay", &totem_config->max_network_delay);
-
-	objdb_get_int (objdb,object_totem_handle, "window_size", &totem_config->window_size);
-	(void)objdb_get_string (objdb, object_totem_handle, "vsftype", &totem_config->vsf_type);
-
-	objdb_get_int (objdb,object_totem_handle, "max_messages", &totem_config->max_messages);
-
-	objdb_get_int (objdb,object_totem_handle, "miss_count_const", &totem_config->miss_count_const);
+	icmap_iter_finalize(iter);
 }
 
-
-static void totem_get_crypto_type(
-	const struct objdb_iface_ver0 *objdb,
-	hdb_handle_t object_totem_handle,
-	struct totem_config *totem_config)
+static void config_convert_nodelist_to_interface(struct totem_config *totem_config)
 {
-	const char *str;
+	icmap_iter_t iter;
+	const char *iter_key;
+	int res = 0;
+	int node_pos;
+	char tmp_key[ICMAP_KEYNAME_MAXLEN];
+	char tmp_key2[ICMAP_KEYNAME_MAXLEN];
+	char *node_addr_str;
+	unsigned int ringnumber = 0;
+	struct list_head addrs;
+	struct list_head *list;
+	struct totem_ip_if_address *if_addr;
+	struct totem_ip_address node_addr;
+	int node_found;
 
-	totem_config->crypto_accept = TOTEM_CRYPTO_ACCEPT_OLD;
-	if (!objdb_get_string (objdb, object_totem_handle, "crypto_accept", &str)) {
-		if (strcmp(str, "new") == 0) {
-			totem_config->crypto_accept = TOTEM_CRYPTO_ACCEPT_NEW;
+	if (totemip_getifaddrs(&addrs) == -1) {
+		return ;
+	}
+
+	iter = icmap_iter_init("nodelist.node.");
+	while ((iter_key = icmap_iter_next(iter, NULL, NULL)) != NULL) {
+		res = sscanf(iter_key, "nodelist.node.%u.%s", &node_pos, tmp_key);
+		if (res != 2) {
+			continue;
+		}
+
+		if (strcmp(tmp_key, "ring0_addr") != 0) {
+			continue;
+		}
+
+		if (icmap_get_string(iter_key, &node_addr_str) != CS_OK) {
+			continue ;
+		}
+
+		if (totemip_parse(&node_addr, node_addr_str, totem_config->ip_version) == -1) {
+			free(node_addr_str);
+			continue ;
+		}
+		free(node_addr_str);
+
+		/*
+		 * Try to find node in if_addrs
+		 */
+		node_found = 0;
+		for (list = addrs.next; list != &addrs; list = list->next) {
+			if_addr = list_entry(list, struct totem_ip_if_address, list);
+
+			if (totemip_equal(&node_addr, &if_addr->ip_addr)) {
+				node_found = 1;
+				break;
+			}
+		}
+
+		if (node_found) {
+			break ;
 		}
 	}
 
-	totem_config->crypto_type = TOTEM_CRYPTO_SOBER;
+	icmap_iter_finalize(iter);
 
-#ifdef HAVE_LIBNSS
-	/*
-	 * We must set these even if the key does not exist.
-	 * Encryption type can be set on-the-fly using CFG
-	 */
-	totem_config->crypto_crypt_type = CKM_AES_CBC_PAD;
-	totem_config->crypto_sign_type = CKM_SHA256_RSA_PKCS;
-#endif
+	if (node_found) {
+		/*
+		 * We found node, so create interface section
+		 */
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.", node_pos);
+		iter = icmap_iter_init(tmp_key);
+		while ((iter_key = icmap_iter_next(iter, NULL, NULL)) != NULL) {
+			res = sscanf(iter_key, "nodelist.node.%u.ring%u%s", &node_pos, &ringnumber, tmp_key2);
+			if (res != 3 || strcmp(tmp_key2, "_addr") != 0) {
+				continue ;
+			}
 
-	if (!objdb_get_string (objdb, object_totem_handle, "crypto_type", &str)) {
-		if (strcmp(str, "sober") == 0) {
-			return;
+			if (icmap_get_string(iter_key, &node_addr_str) != CS_OK) {
+				continue;
+			}
+
+			snprintf(tmp_key2, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.bindnetaddr", ringnumber);
+			icmap_set_string(tmp_key2, node_addr_str);
+			free(node_addr_str);
 		}
-#ifdef HAVE_LIBNSS
-		if (strcmp(str, "nss") == 0) {
-			totem_config->crypto_type = TOTEM_CRYPTO_NSS;
-
-		}
-#endif
+		icmap_iter_finalize(iter);
 	}
 }
 
 
-
 extern int totem_config_read (
-	struct objdb_iface_ver0 *objdb,
 	struct totem_config *totem_config,
-	const char **error_string)
+	const char **error_string,
+	uint64_t *warnings)
 {
 	int res = 0;
-	hdb_handle_t object_totem_handle;
-	hdb_handle_t object_interface_handle;
-	hdb_handle_t object_member_handle;
-	const char *str;
+	char *str;
 	unsigned int ringnumber = 0;
-	hdb_handle_t object_find_interface_handle;
-	hdb_handle_t object_find_member_handle;
-	const char *transport_type;
 	int member_count = 0;
+	icmap_iter_t iter, member_iter;
+	const char *iter_key;
+	const char *member_iter_key;
+	char ringnumber_key[ICMAP_KEYNAME_MAXLEN];
+	char tmp_key[ICMAP_KEYNAME_MAXLEN];
+	uint8_t u8;
+	uint16_t u16;
+	char *cluster_name = NULL;
+	int i;
+	int local_node_pos;
+	int nodeid_set;
 
-	res = totem_handle_find (objdb, &object_totem_handle);
-	if (res == -1) {
-printf ("couldn't find totem handle\n");
-		return (-1);
-	}
+	*warnings = 0;
 
 	memset (totem_config, 0, sizeof (struct totem_config));
 	totem_config->interfaces = malloc (sizeof (struct totem_interface) * INTERFACE_MAX);
@@ -302,68 +457,75 @@ printf ("couldn't find totem handle\n");
 	memset (totem_config->interfaces, 0,
 		sizeof (struct totem_interface) * INTERFACE_MAX);
 
-	totem_config->secauth = 1;
-
 	strcpy (totem_config->rrp_mode, "none");
 
-	if (!objdb_get_string (objdb, object_totem_handle, "version", &str)) {
-		if (strcmp (str, "2") == 0) {
-			totem_config->version = 2;
-		}
-	}
-	if (!objdb_get_string (objdb, object_totem_handle, "secauth", &str)) {
-		if (strcmp (str, "on") == 0) {
-			totem_config->secauth = 1;
-		}
-		if (strcmp (str, "off") == 0) {
-			totem_config->secauth = 0;
-		}
+	icmap_get_uint32("totem.version", (uint32_t *)&totem_config->version);
+
+	if (totem_get_crypto(totem_config) != 0) {
+		*error_string = "crypto_cipher requires crypto_hash with value other than none";
+		return -1;
 	}
 
-	if (totem_config->secauth == 1) {
-		totem_get_crypto_type(objdb, object_totem_handle, totem_config);
-	}
-
-	if (!objdb_get_string (objdb, object_totem_handle, "rrp_mode", &str)) {
+	if (icmap_get_string("totem.rrp_mode", &str) == CS_OK) {
 		strcpy (totem_config->rrp_mode, str);
+		free(str);
 	}
 
-	/*
-	 * Get interface node id
-	 */
-	objdb_get_int (objdb, object_totem_handle, "nodeid", &totem_config->node_id);
+	icmap_get_uint32("totem.nodeid", &totem_config->node_id);
 
 	totem_config->clear_node_high_bit = 0;
-	if (!objdb_get_string (objdb,object_totem_handle, "clear_node_high_bit", &str)) {
+	if (icmap_get_string("totem.clear_node_high_bit", &str) == CS_OK) {
 		if (strcmp (str, "yes") == 0) {
 			totem_config->clear_node_high_bit = 1;
 		}
+		free(str);
 	}
 
-	objdb_get_int (objdb,object_totem_handle, "threads", &totem_config->threads);
+	icmap_get_uint32("totem.threads", &totem_config->threads);
 
+	icmap_get_uint32("totem.netmtu", &totem_config->net_mtu);
 
-	objdb_get_int (objdb,object_totem_handle, "netmtu", &totem_config->net_mtu);
+	icmap_get_string("totem.cluster_name", &cluster_name);
+
+	totem_config->ip_version = AF_INET;
+	if (icmap_get_string("totem.ip_version", &str) == CS_OK) {
+		if (strcmp(str, "ipv4") == 0) {
+			totem_config->ip_version = AF_INET;
+		}
+		if (strcmp(str, "ipv6") == 0) {
+			totem_config->ip_version = AF_INET6;
+		}
+		free(str);
+	}
 
 	/*
 	 * Get things that might change in the future
 	 */
-	totem_volatile_config_read (objdb, totem_config, object_totem_handle);
+	totem_volatile_config_read(totem_config);
 
-	objdb->object_find_create (
-		object_totem_handle,
-		"interface",
-		strlen ("interface"),
-		&object_find_interface_handle);
+	if (icmap_get_string("totem.interface.0.bindnetaddr", &str) != CS_OK) {
+		/*
+		 * We were not able to find ring 0 bindnet addr. Try to use nodelist informations
+		 */
+		config_convert_nodelist_to_interface(totem_config);
+	} else {
+		free(str);
+	}
 
-	while (objdb->object_find_next (
-		object_find_interface_handle,
-		&object_interface_handle) == 0) {
+	iter = icmap_iter_init("totem.interface.");
+	while ((iter_key = icmap_iter_next(iter, NULL, NULL)) != NULL) {
+		res = sscanf(iter_key, "totem.interface.%[^.].%s", ringnumber_key, tmp_key);
+		if (res != 2) {
+			continue;
+		}
+
+		if (strcmp(tmp_key, "bindnetaddr") != 0) {
+			continue;
+		}
 
 		member_count = 0;
 
-		objdb_get_int (objdb, object_interface_handle, "ringnumber", &ringnumber);
-
+		ringnumber = atoi(ringnumber_key);
 
 		if (ringnumber >= INTERFACE_MAX) {
 			snprintf (error_string_response, sizeof(error_string_response),
@@ -374,84 +536,160 @@ printf ("couldn't find totem handle\n");
 			return -1;
 		}
 
+		/*
+		 * Get the bind net address
+		 */
+		if (icmap_get_string(iter_key, &str) == CS_OK) {
+			res = totemip_parse (&totem_config->interfaces[ringnumber].bindnet, str,
+						     totem_config->interfaces[ringnumber].mcast_addr.family);
+			free(str);
+		}
+
 		/*
 		 * Get interface multicast address
 		 */
-		if (!objdb_get_string (objdb, object_interface_handle, "mcastaddr", &str)) {
-			res = totemip_parse (&totem_config->interfaces[ringnumber].mcast_addr, str, 0);
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.mcastaddr", ringnumber);
+		if (icmap_get_string(tmp_key, &str) == CS_OK) {
+			res = totemip_parse (&totem_config->interfaces[ringnumber].mcast_addr, str, totem_config->ip_version);
+			free(str);
+		} else {
+			/*
+			 * User not specified address -> autogenerate one from cluster_name key
+			 * (if available)
+			 */
+			res = get_cluster_mcast_addr (cluster_name,
+					&totem_config->interfaces[ringnumber].bindnet,
+					ringnumber,
+					totem_config->ip_version,
+					&totem_config->interfaces[ringnumber].mcast_addr);
 		}
+
 		totem_config->broadcast_use = 0;
-		if (!objdb_get_string (objdb, object_interface_handle, "broadcast", &str)) {
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.broadcast", ringnumber);
+		if (icmap_get_string(tmp_key, &str) == CS_OK) {
 			if (strcmp (str, "yes") == 0) {
 				totem_config->broadcast_use = 1;
 				totemip_parse (
 					&totem_config->interfaces[ringnumber].mcast_addr,
-					"255.255.255.255", 0);
+					"255.255.255.255", totem_config->ip_version);
 			}
+			free(str);
 		}
 
 		/*
 		 * Get mcast port
 		 */
-		if (!objdb_get_string (objdb, object_interface_handle, "mcastport", &str)) {
-			totem_config->interfaces[ringnumber].ip_port = atoi (str);
-		}
-
-		/*
-		 * Get the bind net address
-		 */
-		if (!objdb_get_string (objdb, object_interface_handle, "bindnetaddr", &str)) {
-
-			res = totemip_parse (&totem_config->interfaces[ringnumber].bindnet, str,
-					     totem_config->interfaces[ringnumber].mcast_addr.family);
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.mcastport", ringnumber);
+		if (icmap_get_uint16(tmp_key, &totem_config->interfaces[ringnumber].ip_port) != CS_OK) {
+			if (totem_config->broadcast_use) {
+				totem_config->interfaces[ringnumber].ip_port = DEFAULT_PORT + (2 * ringnumber);
+			} else {
+				totem_config->interfaces[ringnumber].ip_port = DEFAULT_PORT;
+			}
 		}
 
 		/*
 		 * Get the TTL
 		 */
 		totem_config->interfaces[ringnumber].ttl = 1;
-		if (!objdb_get_string (objdb, object_interface_handle, "ttl", &str)) {
-			totem_config->interfaces[ringnumber].ttl = atoi (str);
+
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.ttl", ringnumber);
+
+		if (icmap_get_uint8(tmp_key, &u8) == CS_OK) {
+			totem_config->interfaces[ringnumber].ttl = u8;
 		}
 
-		objdb->object_find_create (
-			object_interface_handle,
-			"member",
-			strlen ("member"),
-			&object_find_member_handle);
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.member.", ringnumber);
+		member_iter = icmap_iter_init(tmp_key);
+		while ((member_iter_key = icmap_iter_next(member_iter, NULL, NULL)) != NULL) {
+			if (member_count == 0) {
+				if (icmap_get_string("nodelist.node.0.ring0_addr", &str) == CS_OK) {
+					free(str);
+					*warnings |= TOTEM_CONFIG_WARNING_MEMBERS_IGNORED;
+					break;
+				} else {
+					*warnings |= TOTEM_CONFIG_WARNING_MEMBERS_DEPRECATED;
+				}
+			}
 
-		while (objdb->object_find_next (
-			object_find_member_handle,
-			&object_member_handle) == 0) {
-
-			if (!objdb_get_string (objdb, object_member_handle, "memberaddr", &str)) {
-				res = totemip_parse (&totem_config->interfaces[ringnumber].member_list[member_count++], str, 0);
+			if (icmap_get_string(member_iter_key, &str) == CS_OK) {
+				res = totemip_parse (&totem_config->interfaces[ringnumber].member_list[member_count++],
+						str, totem_config->ip_version);
 			}
-		
 		}
+		icmap_iter_finalize(member_iter);
+
 		totem_config->interfaces[ringnumber].member_count = member_count;
 		totem_config->interface_count++;
-		objdb->object_find_destroy (object_find_member_handle);
 	}
+	icmap_iter_finalize(iter);
 
-	objdb->object_find_destroy (object_find_interface_handle);
+	/*
+	 * Store automatically generated items back to icmap
+	 */
+	for (i = 0; i < totem_config->interface_count; i++) {
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.mcastaddr", i);
+		if (icmap_get_string(tmp_key, &str) == CS_OK) {
+			free(str);
+		} else {
+			str = (char *)totemip_print(&totem_config->interfaces[i].mcast_addr);
+			icmap_set_string(tmp_key, str);
+		}
 
-	add_totem_config_notification(objdb, totem_config, object_totem_handle);
+		snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "totem.interface.%u.mcastport", i);
+		if (icmap_get_uint16(tmp_key, &u16) != CS_OK) {
+			icmap_set_uint16(tmp_key, totem_config->interfaces[i].ip_port);
+		}
+	}
 
 	totem_config->transport_number = TOTEM_TRANSPORT_UDP;
-	(void)objdb_get_string (objdb, object_totem_handle, "transport", &transport_type);
-
-	if (transport_type) {
-		if (strcmp (transport_type, "udpu") == 0) {
+	if (icmap_get_string("totem.transport", &str) == CS_OK) {
+		if (strcmp (str, "udpu") == 0) {
 			totem_config->transport_number = TOTEM_TRANSPORT_UDPU;
 		}
-	}
-	if (transport_type) {
-		if (strcmp (transport_type, "iba") == 0) {
+
+		if (strcmp (str, "iba") == 0) {
 			totem_config->transport_number = TOTEM_TRANSPORT_RDMA;
 		}
+		free(str);
 	}
 
+	free(cluster_name);
+
+	/*
+	 * Check existence of nodelist
+	 */
+	if (icmap_get_string("nodelist.node.0.ring0_addr", &str) == CS_OK) {
+		free(str);
+		/*
+		 * find local node
+		 */
+		local_node_pos = find_local_node_in_nodelist(totem_config);
+		if (local_node_pos != -1) {
+			icmap_set_uint32("nodelist.local_node_pos", local_node_pos);
+
+			snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.nodeid", local_node_pos);
+
+			nodeid_set = (totem_config->node_id != 0);
+			if (icmap_get_uint32(tmp_key, &totem_config->node_id) == CS_OK && nodeid_set) {
+				*warnings |= TOTEM_CONFIG_WARNING_TOTEM_NODEID_IGNORED;
+			}
+
+			/*
+			 * Make localnode ring0_addr read only, so we can be sure that local
+			 * node never changes. If rebinding to other IP would be in future
+			 * supported, this must be changed and handled properly!
+			 */
+			snprintf(tmp_key, ICMAP_KEYNAME_MAXLEN, "nodelist.node.%u.ring0_addr", local_node_pos);
+			icmap_set_ro_access(tmp_key, 0, 1);
+			icmap_set_ro_access("nodelist.local_node_pos", 0, 1);
+		}
+
+		put_nodelist_members_to_config(totem_config);
+	}
+
+	add_totem_config_notification(totem_config);
+
 	return 0;
 }
 
@@ -540,20 +778,6 @@ int totem_config_validate (
 	 */
 	if (totem_config->token_timeout == 0) {
 		totem_config->token_timeout = TOKEN_TIMEOUT;
-		if (totem_config->token_retransmits_before_loss_const == 0) {
-			totem_config->token_retransmits_before_loss_const = TOKEN_RETRANSMITS_BEFORE_LOSS_CONST;
-		}
-
-		if (totem_config->token_retransmit_timeout == 0) {
-			totem_config->token_retransmit_timeout =
-				(int)(totem_config->token_timeout /
-				(totem_config->token_retransmits_before_loss_const + 0.2));
-		}
-		if (totem_config->token_hold_timeout == 0) {
-			totem_config->token_hold_timeout =
-				(int)(totem_config->token_retransmit_timeout * 0.8 -
-				(1000/HZ));
-		}
 	}
 
 	if (totem_config->max_network_delay == 0) {
@@ -603,10 +827,6 @@ int totem_config_validate (
 		goto parse_error;
 	}
 
-	if (totem_config->token_hold_timeout == 0) {
-		totem_config->token_hold_timeout = TOKEN_HOLD_TIMEOUT;
-	}
-
 	if (totem_config->token_hold_timeout < MINIMUM_TIMEOUT) {
 		snprintf (local_error_reason, sizeof(local_error_reason),
 			"The token hold timeout parameter (%d ms) may not be less then (%d ms).",
@@ -744,9 +964,6 @@ int totem_config_validate (
 	if (totem_config->threads > SEND_THREADS_MAX) {
 		totem_config->threads = SEND_THREADS_MAX;
 	}
-	if (totem_config->secauth == 0) {
-		totem_config->threads = 0;
-	}
 	if (totem_config->net_mtu > FRAME_SIZE_MAX) {
 		error_reason = "This net_mtu parameter is greater then the maximum frame size";
 		goto parse_error;
@@ -778,7 +995,7 @@ static int read_keyfile (
 
 	fd = open (key_location, O_RDONLY);
 	if (fd == -1) {
-		LOGSYS_STRERROR_R (error_ptr, errno, error_str, sizeof(error_str));
+		error_ptr = qb_strerror_r(errno, error_str, sizeof(error_str));
 		snprintf (error_string_response, sizeof(error_string_response),
 			"Could not open %s: %s\n",
 			 key_location, error_ptr);
@@ -790,7 +1007,7 @@ static int read_keyfile (
 	close (fd);
 
 	if (res == -1) {
-		LOGSYS_STRERROR_R (error_ptr, saved_errno, error_str, sizeof(error_str));
+		error_ptr = qb_strerror_r (saved_errno, error_str, sizeof(error_str));
 		snprintf (error_string_response, sizeof(error_string_response),
 			"Could not read %s: %s\n",
 			 key_location, error_ptr);
@@ -814,50 +1031,43 @@ parse_error:
 }
 
 int totem_config_keyread (
-	struct objdb_iface_ver0 *objdb,
 	struct totem_config *totem_config,
 	const char **error_string)
 {
 	int got_key = 0;
-	const char *key_location = NULL;
-	hdb_handle_t object_totem_handle;
+	char *key_location = NULL;
 	int res;
+	size_t key_len;
 
 	memset (totem_config->private_key, 0, 128);
 	totem_config->private_key_len = 128;
 
-	if (totem_config->secauth == 0) {
+	if (strcmp(totem_config->crypto_cipher_type, "none") == 0 &&
+	    strcmp(totem_config->crypto_hash_type, "none") == 0) {
 		return (0);
 	}
 
-	res = totem_handle_find (objdb, &object_totem_handle);
-	if (res == -1) {
-		return (-1);
-	}
-	/* objdb may store the location of the key file */
-	if (!objdb_get_string (objdb,object_totem_handle, "keyfile", &key_location)
-	    && key_location) {
+	/* cmap may store the location of the key file */
+	if (icmap_get_string("totem.keyfile", &key_location) == CS_OK) {
 		res = read_keyfile(key_location, totem_config, error_string);
+		free(key_location);
 		if (res)  {
 			goto key_error;
 		}
 		got_key = 1;
-	} else { /* Or the key itself may be in the objdb */
-		char *key = NULL;
-		size_t key_len;
-		res = objdb->object_key_get (object_totem_handle,
-			"key",
-			strlen ("key"),
-			(void *)&key,
-			&key_len);
-
-		if (res == 0 && key) {
+	} else { /* Or the key itself may be in the cmap */
+		if (icmap_get("totem.key", NULL, &key_len, NULL) == CS_OK) {
 			if (key_len > sizeof (totem_config->private_key)) {
+				sprintf(error_string_response, "key is too long");
+				goto key_error;
+			}
+			if (icmap_get("totem.key", totem_config->private_key, &key_len, NULL) == CS_OK) {
+				totem_config->private_key_len = key_len;
+				got_key = 1;
+			} else {
+				sprintf(error_string_response, "can't store private key");
 				goto key_error;
 			}
-			memcpy(totem_config->private_key, key, key_len);
-			totem_config->private_key_len = key_len;
-			got_key = 1;
 		}
 	}
 
@@ -880,100 +1090,23 @@ key_error:
 
 }
 
-static void totem_key_change_notify(object_change_type_t change_type,
-			      hdb_handle_t parent_object_handle,
-			      hdb_handle_t object_handle,
-			      const void *object_name_pt, size_t object_name_len,
-			      const void *key_name_pt, size_t key_len,
-			      const void *key_value_pt, size_t key_value_len,
-			      void *priv_data_pt)
+static void totem_change_notify(
+	int32_t event,
+	const char *key_name,
+	struct icmap_notify_value new_val,
+	struct icmap_notify_value old_val,
+	void *user_data)
 {
-	struct totem_config *totem_config = priv_data_pt;
-
-	if (memcmp(object_name_pt, "totem", object_name_len) == 0)
-		totem_volatile_config_read(global_objdb,
-					   totem_config,
-					   object_handle); // CHECK
+	totem_volatile_config_read((struct totem_config *)user_data);
 }
 
-static void totem_objdb_reload_notify(objdb_reload_notify_type_t type, int flush,
-				      void *priv_data_pt)
+static void add_totem_config_notification(struct totem_config *totem_config)
 {
-	struct totem_config *totem_config = priv_data_pt;
-	hdb_handle_t totem_object_handle;
-
-	if (totem_config == NULL)
-	        return;
-
-	/*
-	 * A new totem {} key might exist, cancel the
-	 * existing notification at the start of reload,
-	 * and start a new one on the new object when
-	 * it's all settled.
-	 */
-
-	if (type == OBJDB_RELOAD_NOTIFY_START) {
-		global_objdb->object_track_stop(
-			totem_key_change_notify,
-			NULL,
-			NULL,
-			NULL,
-			totem_config);
-	}
-
-	if (type == OBJDB_RELOAD_NOTIFY_END ||
-	    type == OBJDB_RELOAD_NOTIFY_FAILED) {
-
-
-		if (!totem_handle_find(global_objdb,
-				      &totem_object_handle)) {
-
-		        global_objdb->object_track_start(totem_object_handle,
-						  1,
-						  totem_key_change_notify,
-						  NULL, // object_create_notify,
-						  NULL, // object_destroy_notify,
-						  NULL, // object_reload_notify
-						  totem_config); // priv_data
-			/*
-			 * Reload the configuration
-			 */
-			totem_volatile_config_read(global_objdb,
-						   totem_config,
-						   totem_object_handle);
-
-		}
-		else {
-			log_printf(LOGSYS_LEVEL_ERROR, "totem objdb tracking stopped, cannot find totem{} handle on objdb\n");
-		}
-	}
-}
-
-
-static void add_totem_config_notification(
-	struct objdb_iface_ver0 *objdb,
-	struct totem_config *totem_config,
-	hdb_handle_t totem_object_handle)
-{
-
-	global_objdb = objdb;
-	objdb->object_track_start(totem_object_handle,
-				  1,
-				  totem_key_change_notify,
-				  NULL, // object_create_notify,
-				  NULL, // object_destroy_notify,
-				  NULL, // object_reload_notify
-				  totem_config); // priv_data
-
-	/*
-	 * Reload notify must be on the parent object
-	 */
-	objdb->object_track_start(OBJECT_PARENT_HANDLE,
-				  1,
-				  NULL, // key_change_notify,
-				  NULL, // object_create_notify,
-				  NULL, // object_destroy_notify,
-				  totem_objdb_reload_notify, // object_reload_notify
-				  totem_config); // priv_data
+	icmap_track_t icmap_track;
 
+	icmap_track_add("totem.",
+		ICMAP_TRACK_ADD | ICMAP_TRACK_DELETE | ICMAP_TRACK_MODIFY | ICMAP_TRACK_PREFIX,
+		totem_change_notify,
+		totem_config,
+		&icmap_track);
 }
