@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2003-2005 MontaVista Software, Inc.
  * Copyright (c) 2005 OSDL.
- * Copyright (c) 2006-2009 Red Hat, Inc.
+ * Copyright (c) 2006-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -87,7 +87,10 @@
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
 #endif
+#include <sys/types.h>
+#include <sys/socket.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <sys/uio.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -98,12 +101,12 @@
 #include <limits.h>
 
 #include <corosync/swab.h>
-#include <corosync/hdb.h>
 #include <corosync/list.h>
-#include <corosync/totem/coropoll.h>
+#include <qb/qbloop.h>
+#include <qb/qbipcs.h>
 #include <corosync/totem/totempg.h>
 #define LOGSYS_UTILS_ONLY 1
-#include <corosync/engine/logsys.h>
+#include <corosync/logsys.h>
 
 #include "totemmrp.h"
 #include "totemsrp.h"
@@ -152,6 +155,20 @@ struct totempg_mcast {
 #define TOTEMPG_PACKET_SIZE (totempg_totem_config->net_mtu - \
 	sizeof (struct totempg_mcast))
 
+/*
+ * Local variables used for packing small messages
+ */
+static unsigned short mcast_packed_msg_lens[FRAME_SIZE_MAX];
+
+static int mcast_packed_msg_count = 0;
+
+static int totempg_reserved = 1;
+
+static unsigned int totempg_size_limit;
+
+static totem_queue_level_changed_fn totem_queue_level_changed = NULL;
+
+static uint32_t totempg_threaded_mode = 0;
 
 /*
  * Function and data used to log messages
@@ -163,11 +180,12 @@ static int totempg_log_level_notice;
 static int totempg_log_level_debug;
 static int totempg_subsys_id;
 static void (*totempg_log_printf) (
-	unsigned int rec_ident,
+	int level,
+	int subsys,
 	const char *function,
 	const char *file,
 	int line,
-	const char *format, ...) __attribute__((format(printf, 5, 6)));
+	const char *format, ...) __attribute__((format(printf, 6, 7)));
 
 struct totem_config *totempg_totem_config;
 
@@ -194,48 +212,33 @@ static int callback_token_received_fn (enum totem_callback_token_type type,
 
 DECLARE_LIST_INIT(assembly_list_inuse);
 
+DECLARE_LIST_INIT(assembly_list_free);
+
 DECLARE_LIST_INIT(assembly_list_inuse_trans);
 
 DECLARE_LIST_INIT(assembly_list_free_trans);
 
-DECLARE_LIST_INIT(assembly_list_free);
+DECLARE_LIST_INIT(totempg_groups_list);
 
 /*
- * Structure for storing totem_pg contexts so they can be switched
+ * Staging buffer for packed messages.  Messages are staged in this buffer
+ * before sending.  Multiple messages may fit which cuts down on the
+ * number of mcasts sent.  If a message doesn't completely fit, then
+ * the mcast header has a fragment bit set that says that there are more
+ * data to follow.  fragment_size is an index into the buffer.  It indicates
+ * the size of message data and where to place new message data.
+ * fragment_contuation indicates whether the first packed message in
+ * the buffer is a continuation of a previously packed fragment.
  */
-struct totempg_context {
-	/*
-	 * Staging buffer for packed messages.  Messages are staged in this buffer
-	 * before sending.  Multiple messages may fit which cuts down on the
-	 * number of mcasts sent.  If a message doesn't completely fit, then
-	 * the mcast header has a fragment bit set that says that there are more
-	 * data to follow.  fragment_size is an index into the buffer.  It indicates
-	 * the size of message data and where to place new message data.
-	 * fragment_contuation indicates whether the first packed message in
-	 * the buffer is a continuation of a previously packed fragment.
-	 */
-	unsigned char *fragmentation_data;
+static unsigned char *fragmentation_data;
 
-	int fragment_size;
+static int fragment_size = 0;
 
-	int fragment_continuation;
+static int fragment_continuation = 0;
 
-	unsigned char next_fragment;
+static struct iovec iov_delv;
 
-	/*
-	 * Local variables used for packing small messages
-	 */
-	unsigned short mcast_packed_msg_lens[FRAME_SIZE_MAX];
-
-	int mcast_packed_msg_count;
-
-	int totempg_reserved;
-};
-
-#define TOTEMPG_NO_CONTEXTS	2
-
-static struct totempg_context totempg_contexts_array[TOTEMPG_NO_CONTEXTS];
-static struct totempg_context *totempg_active_context;
+static int totempg_waiting_transack = 0;
 
 struct totempg_group_instance {
 	void (*deliver_fn) (
@@ -254,13 +257,12 @@ struct totempg_group_instance {
 	struct totempg_group *groups;
 
 	int groups_cnt;
+	int32_t q_level;
+
+	struct list_head list;
 };
 
-DECLARE_HDB_DATABASE (totempg_groups_instance_database,NULL);
-
-static unsigned int totempg_max_handle = 0;
-static int totempg_waiting_transack = 0;
-static unsigned int totempg_size_limit = 0;
+static unsigned char next_fragment = 1;
 
 static pthread_mutex_t totempg_mutex = PTHREAD_MUTEX_INITIALIZER;
 
@@ -268,37 +270,24 @@ static pthread_mutex_t callback_token_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static pthread_mutex_t mcast_msg_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-#define log_printf(level, format, args...)				\
-do {									\
-        totempg_log_printf (						\
-		LOGSYS_ENCODE_RECID(level,				\
-				    totempg_subsys_id,			\
-				    LOGSYS_RECID_LOG),			\
-		__FUNCTION__, __FILE__, __LINE__,			\
-		format, ##args);					\
+#define log_printf(level, format, args...)			\
+do {								\
+        totempg_log_printf(level,				\
+			   totempg_subsys_id,			\
+			   __FUNCTION__, __FILE__, __LINE__,	\
+			   format, ##args);			\
 } while (0);
 
 static int msg_count_send_ok (int msg_count);
 
 static int byte_count_send_ok (int byte_count);
 
-static void totempg_set_context(int context_no)
-{
-	totempg_active_context = &totempg_contexts_array[context_no];
-}
-
 static void totempg_waiting_trans_ack_cb (int waiting_trans_ack)
 {
 	log_printf(LOG_DEBUG, "waiting_trans_ack changed to %u", waiting_trans_ack);
 	totempg_waiting_transack = waiting_trans_ack;
-	if (!waiting_trans_ack) {
-		totempg_set_context(0);
-	} else {
-		totempg_set_context(1);
-	}
 }
 
-
 static struct assembly *assembly_ref (unsigned int nodeid)
 {
 	struct assembly *assembly;
@@ -405,6 +394,7 @@ static void assembly_deref_from_normal_and_trans (int nodeid)
 			}
 		}
 	}
+
 }
 
 static inline void app_confchg_fn (
@@ -416,7 +406,7 @@ static inline void app_confchg_fn (
 {
 	int i;
 	struct totempg_group_instance *instance;
-	unsigned int res;
+	struct list_head *list;
 
 	/*
 	 * For every leaving processor, add to free list
@@ -426,25 +416,23 @@ static inline void app_confchg_fn (
 	for (i = 0; i < left_list_entries; i++) {
 		assembly_deref_from_normal_and_trans (left_list[i]);
 	}
-	for (i = 0; i <= totempg_max_handle; i++) {
-		res = hdb_handle_get (&totempg_groups_instance_database,
-			hdb_nocheck_convert (i), (void *)&instance);
 
-		if (res == 0) {
-			if (instance->confchg_fn) {
-				instance->confchg_fn (
-					configuration_type,
-					member_list,
-					member_list_entries,
-					left_list,
-					left_list_entries,
-					joined_list,
-					joined_list_entries,
-					ring_id);
-			}
+	for (list = totempg_groups_list.next;
+		list != &totempg_groups_list;
+		list = list->next) {
 
-			hdb_handle_put (&totempg_groups_instance_database,
-				hdb_nocheck_convert (i));
+		instance = list_entry (list, struct totempg_group_instance, list);
+
+		if (instance->confchg_fn) {
+			instance->confchg_fn (
+				configuration_type,
+				member_list,
+				member_list_entries,
+				left_list,
+				left_list_entries,
+				joined_list,
+				joined_list_entries,
+				ring_id);
 		}
 	}
 }
@@ -546,12 +534,11 @@ static inline void app_deliver_fn (
 	unsigned int msg_len,
 	int endian_conversion_required)
 {
-	int i;
 	struct totempg_group_instance *instance;
 	struct iovec stripped_iovec;
 	unsigned int adjust_iovec;
-	unsigned int res;
 	struct iovec *iovec;
+	struct list_head *list;
 
         struct iovec aligned_iovec = { NULL, 0 };
 
@@ -579,38 +566,35 @@ static inline void app_deliver_fn (
 
 	iovec = &aligned_iovec;
 
-	for (i = 0; i <= totempg_max_handle; i++) {
-		res = hdb_handle_get (&totempg_groups_instance_database,
-			hdb_nocheck_convert (i), (void *)&instance);
+	for (list = totempg_groups_list.next;
+		list != &totempg_groups_list;
+		list = list->next) {
 
-		if (res == 0) {
-			if (group_matches (iovec, 1, instance->groups, instance->groups_cnt, &adjust_iovec)) {
-				stripped_iovec.iov_len = iovec->iov_len - adjust_iovec;
-				stripped_iovec.iov_base = (char *)iovec->iov_base + adjust_iovec;
+		instance = list_entry (list, struct totempg_group_instance, list);
+		if (group_matches (iovec, 1, instance->groups, instance->groups_cnt, &adjust_iovec)) {
+			stripped_iovec.iov_len = iovec->iov_len - adjust_iovec;
+			stripped_iovec.iov_base = (char *)iovec->iov_base + adjust_iovec;
 
 #ifdef TOTEMPG_NEED_ALIGN
+			/*
+			 * Align data structure for not i386 or x86_64
+			 */
+			if ((char *)iovec->iov_base + adjust_iovec % 4 != 0) {
 				/*
-				 * Align data structure for not i386 or x86_64
+				 * Deal with misalignment
 				 */
-				if ((char *)iovec->iov_base + adjust_iovec % 4 != 0) {
-					/*
-					 * Deal with misalignment
-					 */
-					stripped_iovec.iov_base =
-						alloca (stripped_iovec.iov_len);
-					memcpy (stripped_iovec.iov_base,
-						 (char *)iovec->iov_base + adjust_iovec,
-						stripped_iovec.iov_len);
-				}
-#endif
-				instance->deliver_fn (
-					nodeid,
-					stripped_iovec.iov_base,
-					stripped_iovec.iov_len,
-					endian_conversion_required);
+				stripped_iovec.iov_base =
+					alloca (stripped_iovec.iov_len);
+				memcpy (stripped_iovec.iov_base,
+					 (char *)iovec->iov_base + adjust_iovec,
+					stripped_iovec.iov_len);
 			}
-
-			hdb_handle_put (&totempg_groups_instance_database, hdb_nocheck_convert(i));
+#endif
+			instance->deliver_fn (
+				nodeid,
+				stripped_iovec.iov_base,
+				stripped_iovec.iov_len,
+				endian_conversion_required);
 		}
 	}
 }
@@ -646,7 +630,6 @@ static void totempg_deliver_fn (
 	int start;
 	const char *data;
 	int datasize;
-	struct iovec iov_delv;
 
 	assembly = assembly_ref (nodeid);
 	assert (assembly);
@@ -765,42 +748,49 @@ int callback_token_received_fn (enum totem_callback_token_type type,
 {
 	struct totempg_mcast mcast;
 	struct iovec iovecs[3];
-	int res;
-	struct totempg_context *con = totempg_active_context;
 
-	pthread_mutex_lock (&mcast_msg_mutex);
-	if (con->mcast_packed_msg_count == 0) {
-		pthread_mutex_unlock (&mcast_msg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&mcast_msg_mutex);
+	}
+	if (mcast_packed_msg_count == 0) {
+		if (totempg_threaded_mode == 1) {
+			pthread_mutex_unlock (&mcast_msg_mutex);
+		}
 		return (0);
 	}
 	if (totemmrp_avail() == 0) {
-		pthread_mutex_unlock (&mcast_msg_mutex);
+		if (totempg_threaded_mode == 1) {
+			pthread_mutex_unlock (&mcast_msg_mutex);
+		}
 		return (0);
 	}
 	mcast.header.version = 0;
+	mcast.header.type = 0;
 	mcast.fragmented = 0;
 
 	/*
 	 * Was the first message in this buffer a continuation of a
 	 * fragmented message?
 	 */
-	mcast.continuation = con->fragment_continuation;
-	con->fragment_continuation = 0;
+	mcast.continuation = fragment_continuation;
+	fragment_continuation = 0;
 
-	mcast.msg_count = con->mcast_packed_msg_count;
+	mcast.msg_count = mcast_packed_msg_count;
 
 	iovecs[0].iov_base = (void *)&mcast;
 	iovecs[0].iov_len = sizeof (struct totempg_mcast);
-	iovecs[1].iov_base = (void *)con->mcast_packed_msg_lens;
-	iovecs[1].iov_len = con->mcast_packed_msg_count * sizeof (unsigned short);
-	iovecs[2].iov_base = (void *)&con->fragmentation_data[0];
-	iovecs[2].iov_len = con->fragment_size;
-	res = totemmrp_mcast (iovecs, 3, 0);
+	iovecs[1].iov_base = (void *)mcast_packed_msg_lens;
+	iovecs[1].iov_len = mcast_packed_msg_count * sizeof (unsigned short);
+	iovecs[2].iov_base = (void *)&fragmentation_data[0];
+	iovecs[2].iov_len = fragment_size;
+	(void)totemmrp_mcast (iovecs, 3, 0);
 
-	con->mcast_packed_msg_count = 0;
-	con->fragment_size = 0;
+	mcast_packed_msg_count = 0;
+	fragment_size = 0;
 
-	pthread_mutex_unlock (&mcast_msg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&mcast_msg_mutex);
+	}
 	return (0);
 }
 
@@ -808,12 +798,10 @@ int callback_token_received_fn (enum totem_callback_token_type type,
  * Initialize the totem process group abstraction
  */
 int totempg_initialize (
-	hdb_handle_t poll_handle,
+	qb_loop_t *poll_handle,
 	struct totem_config *totem_config)
 {
 	int res;
-	int i;
-	struct totempg_context *con;
 
 	totempg_totem_config = totem_config;
 	totempg_log_level_security = totem_config->totem_logging_configuration.log_level_security;
@@ -824,16 +812,9 @@ int totempg_initialize (
 	totempg_log_printf = totem_config->totem_logging_configuration.log_printf;
 	totempg_subsys_id = totem_config->totem_logging_configuration.log_subsys_id;
 
-	for (i = 0; i < TOTEMPG_NO_CONTEXTS; i++) {
-		con = &totempg_contexts_array[i];
-		memset(con, 0, sizeof(*con));
-
-		con->fragmentation_data = malloc (TOTEMPG_PACKET_SIZE);
-		if (con->fragmentation_data == 0) {
-			return (-1);
-		}
-		con->totempg_reserved = 1;
-		con->next_fragment = 1;
+	fragmentation_data = malloc (TOTEMPG_PACKET_SIZE);
+	if (fragmentation_data == 0) {
+		return (-1);
 	}
 
 	totemsrp_net_mtu_adjust (totem_config);
@@ -857,14 +838,20 @@ int totempg_initialize (
 		(totempg_totem_config->net_mtu -
 		sizeof (struct totempg_mcast) - 16);
 
+	list_init (&totempg_groups_list);
+
 	return (res);
 }
 
 void totempg_finalize (void)
 {
-	pthread_mutex_lock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
+	}
 	totemmrp_finalize ();
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 }
 
 /*
@@ -885,9 +872,10 @@ static int mcast_msg (
 	int copy_len = 0;
 	int copy_base = 0;
 	int total_size = 0;
-	struct totempg_context *con = totempg_active_context;
 
-	pthread_mutex_lock (&mcast_msg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&mcast_msg_mutex);
+	}
 	totemmrp_event_signal (TOTEM_EVENT_NEW_MSG, 1);
 
 	/*
@@ -903,9 +891,9 @@ static int mcast_msg (
 	iov_len = dest;
 
 	max_packet_size = TOTEMPG_PACKET_SIZE -
-		(sizeof (unsigned short) * (con->mcast_packed_msg_count + 1));
+		(sizeof (unsigned short) * (mcast_packed_msg_count + 1));
 
-	con->mcast_packed_msg_lens[con->mcast_packed_msg_count] = 0;
+	mcast_packed_msg_lens[mcast_packed_msg_count] = 0;
 
 	/*
 	 * Check if we would overwrite new message queue
@@ -915,16 +903,18 @@ static int mcast_msg (
 	}
 
 	if (byte_count_send_ok (total_size + sizeof(unsigned short) *
-		(con->mcast_packed_msg_count)) == 0) {
+		(mcast_packed_msg_count)) == 0) {
 
-		pthread_mutex_unlock (&mcast_msg_mutex);
+		if (totempg_threaded_mode == 1) {
+			pthread_mutex_unlock (&mcast_msg_mutex);
+		}
 		return(-1);
 	}
 
 	mcast.header.version = 0;
 	for (i = 0; i < iov_len; ) {
 		mcast.fragmented = 0;
-		mcast.continuation = con->fragment_continuation;
+		mcast.continuation = fragment_continuation;
 		copy_len = iovec[i].iov_len - copy_base;
 
 		/*
@@ -933,14 +923,14 @@ static int mcast_msg (
 		 * fragment_buffer on exit so that max_packet_size + fragment_size
 		 * doesn't exceed the size of the fragment_buffer on the next call.
 		 */
-		if ((copy_len + con->fragment_size) <
+		if ((copy_len + fragment_size) <
 			(max_packet_size - sizeof (unsigned short))) {
 
-			memcpy (&con->fragmentation_data[con->fragment_size],
+			memcpy (&fragmentation_data[fragment_size],
 				(char *)iovec[i].iov_base + copy_base, copy_len);
-			con->fragment_size += copy_len;
-			con->mcast_packed_msg_lens[con->mcast_packed_msg_count] += copy_len;
-			con->next_fragment = 1;
+			fragment_size += copy_len;
+			mcast_packed_msg_lens[mcast_packed_msg_count] += copy_len;
+			next_fragment = 1;
 			copy_len = 0;
 			copy_base = 0;
 			i++;
@@ -952,18 +942,18 @@ static int mcast_msg (
 		} else {
 			unsigned char *data_ptr;
 
-			copy_len = min(copy_len, max_packet_size - con->fragment_size);
+			copy_len = min(copy_len, max_packet_size - fragment_size);
 			if( copy_len == max_packet_size )
 				data_ptr = (unsigned char *)iovec[i].iov_base + copy_base;
 			else {
-				data_ptr = con->fragmentation_data;
-				memcpy (&con->fragmentation_data[con->fragment_size],
+				data_ptr = fragmentation_data;
+				memcpy (&fragmentation_data[fragment_size],
 				(unsigned char *)iovec[i].iov_base + copy_base, copy_len);
 			}
 
-			memcpy (&con->fragmentation_data[con->fragment_size],
+			memcpy (&fragmentation_data[fragment_size],
 				(unsigned char *)iovec[i].iov_base + copy_base, copy_len);
-			con->mcast_packed_msg_lens[con->mcast_packed_msg_count] += copy_len;
+			mcast_packed_msg_lens[mcast_packed_msg_count] += copy_len;
 
 			/*
 			 * if we're not on the last iovec or the iovec is too large to
@@ -972,25 +962,25 @@ static int mcast_msg (
 			 */
 			if ((i < (iov_len - 1)) ||
 					((copy_base + copy_len) < iovec[i].iov_len)) {
-				if (!con->next_fragment) {
-					con->next_fragment++;
+				if (!next_fragment) {
+					next_fragment++;
 				}
-				con->fragment_continuation = con->next_fragment;
-				mcast.fragmented = con->next_fragment++;
-				assert(con->fragment_continuation != 0);
+				fragment_continuation = next_fragment;
+				mcast.fragmented = next_fragment++;
+				assert(fragment_continuation != 0);
 				assert(mcast.fragmented != 0);
 			} else {
-				con->fragment_continuation = 0;
+				fragment_continuation = 0;
 			}
 
 			/*
 			 * assemble the message and send it
 			 */
-			mcast.msg_count = ++con->mcast_packed_msg_count;
+			mcast.msg_count = ++mcast_packed_msg_count;
 			iovecs[0].iov_base = (void *)&mcast;
 			iovecs[0].iov_len = sizeof(struct totempg_mcast);
-			iovecs[1].iov_base = (void *)con->mcast_packed_msg_lens;
-			iovecs[1].iov_len = con->mcast_packed_msg_count *
+			iovecs[1].iov_base = (void *)mcast_packed_msg_lens;
+			iovecs[1].iov_len = mcast_packed_msg_count *
 				sizeof(unsigned short);
 			iovecs[2].iov_base = (void *)data_ptr;
 			iovecs[2].iov_len = max_packet_size;
@@ -1003,9 +993,9 @@ static int mcast_msg (
 			/*
 			 * Recalculate counts and indexes for the next.
 			 */
-			con->mcast_packed_msg_lens[0] = 0;
-			con->mcast_packed_msg_count = 0;
-			con->fragment_size = 0;
+			mcast_packed_msg_lens[0] = 0;
+			mcast_packed_msg_count = 0;
+			fragment_size = 0;
 			max_packet_size = TOTEMPG_PACKET_SIZE - (sizeof(unsigned short));
 
 			/*
@@ -1030,12 +1020,14 @@ static int mcast_msg (
 	 * the last buffer just fit into the fragmentation_data buffer
 	 * and we were at the last iovec.
 	 */
-	if (con->mcast_packed_msg_lens[con->mcast_packed_msg_count]) {
-			con->mcast_packed_msg_count++;
+	if (mcast_packed_msg_lens[mcast_packed_msg_count]) {
+			mcast_packed_msg_count++;
 	}
 
 error_exit:
-	pthread_mutex_unlock (&mcast_msg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&mcast_msg_mutex);
+	}
 	return (res);
 }
 
@@ -1046,12 +1038,11 @@ static int msg_count_send_ok (
 	int msg_count)
 {
 	int avail = 0;
-	struct totempg_context *con = totempg_active_context;
 
 	avail = totemmrp_avail ();
 	totempg_stats.msg_queue_avail = avail;
 
-	return ((avail - con->totempg_reserved) > msg_count);
+	return ((avail - totempg_reserved) > msg_count);
 }
 
 static int byte_count_send_ok (
@@ -1071,11 +1062,10 @@ static int send_reserve (
 	int msg_size)
 {
 	unsigned int msg_count = 0;
-	struct totempg_context *con = totempg_active_context;
 
 	msg_count = (msg_size / (totempg_totem_config->net_mtu - sizeof (struct totempg_mcast) - 16)) + 1;
-	con->totempg_reserved += msg_count;
-	totempg_stats.msg_reserved = con->totempg_reserved;
+	totempg_reserved += msg_count;
+	totempg_stats.msg_reserved = totempg_reserved;
 
 	return (msg_count);
 }
@@ -1083,10 +1073,18 @@ static int send_reserve (
 static void send_release (
 	int msg_count)
 {
-	struct totempg_context *con = totempg_active_context;
+	totempg_reserved -= msg_count;
+	totempg_stats.msg_reserved = totempg_reserved;
+}
 
-	con->totempg_reserved -= msg_count;
-	totempg_stats.msg_reserved = con->totempg_reserved;
+#ifndef HAVE_SMALL_MEMORY_FOOTPRINT
+#undef MESSAGE_QUEUE_MAX
+#define MESSAGE_QUEUE_MAX	((4 * MESSAGE_SIZE_MAX) / totempg_totem_config->net_mtu)
+#endif /* HAVE_SMALL_MEMORY_FOOTPRINT */
+
+static uint32_t q_level_precent_used(void)
+{
+	return (100 - (((totemmrp_avail() - totempg_reserved) * 100) / MESSAGE_QUEUE_MAX));
 }
 
 int totempg_callback_token_create (
@@ -1097,19 +1095,27 @@ int totempg_callback_token_create (
 	const void *data)
 {
 	unsigned int res;
-	pthread_mutex_lock (&callback_token_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&callback_token_mutex);
+	}
 	res = totemmrp_callback_token_create (handle_out, type, delete,
 		callback_fn, data);
-	pthread_mutex_unlock (&callback_token_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&callback_token_mutex);
+	}
 	return (res);
 }
 
 void totempg_callback_token_destroy (
 	void *handle_out)
 {
-	pthread_mutex_lock (&callback_token_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&callback_token_mutex);
+	}
 	totemmrp_callback_token_destroy (handle_out);
-	pthread_mutex_unlock (&callback_token_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&callback_token_mutex);
+	}
 }
 
 /*
@@ -1117,7 +1123,7 @@ void totempg_callback_token_destroy (
  */
 
 int totempg_groups_initialize (
-	hdb_handle_t *handle,
+	void **totempg_groups_instance,
 
 	void (*deliver_fn) (
 		unsigned int nodeid,
@@ -1133,59 +1139,50 @@ int totempg_groups_initialize (
 		const struct memb_ring_id *ring_id))
 {
 	struct totempg_group_instance *instance;
-	unsigned int res;
 
-	pthread_mutex_lock (&totempg_mutex);
-	res = hdb_handle_create (&totempg_groups_instance_database,
-		sizeof (struct totempg_group_instance), handle);
-	if (res != 0) {
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
+	}
+	
+	instance = malloc (sizeof (struct totempg_group_instance));
+	if (instance == NULL) {
 		goto error_exit;
 	}
 
-	if (*handle > totempg_max_handle) {
-		totempg_max_handle = *handle;
-	}
-
-	res = hdb_handle_get (&totempg_groups_instance_database, *handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_destroy;
-	}
-
 	instance->deliver_fn = deliver_fn;
 	instance->confchg_fn = confchg_fn;
 	instance->groups = 0;
 	instance->groups_cnt = 0;
+	instance->q_level = QB_LOOP_MED;
+	list_init (&instance->list);
+	list_add (&instance->list, &totempg_groups_list);
 
-
-	hdb_handle_put (&totempg_groups_instance_database, *handle);
-
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
+	*totempg_groups_instance = instance;
 	return (0);
-error_destroy:
-	hdb_handle_destroy (&totempg_groups_instance_database, *handle);
 
 error_exit:
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 	return (-1);
 }
 
 int totempg_groups_join (
-	hdb_handle_t handle,
+	void *totempg_groups_instance,
 	const struct totempg_group *groups,
 	size_t group_cnt)
 {
-	struct totempg_group_instance *instance;
+	struct totempg_group_instance *instance = (struct totempg_group_instance *)totempg_groups_instance;
 	struct totempg_group *new_groups;
-	unsigned int res;
+	unsigned int res = 0;
 
-	pthread_mutex_lock (&totempg_mutex);
-	res = hdb_handle_get (&totempg_groups_instance_database, handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_exit;
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
 	}
-
+	
 	new_groups = realloc (instance->groups,
 		sizeof (struct totempg_group) *
 		(instance->groups_cnt + group_cnt));
@@ -1198,57 +1195,47 @@ int totempg_groups_join (
 	instance->groups = new_groups;
 	instance->groups_cnt += group_cnt;
 
-	hdb_handle_put (&totempg_groups_instance_database, handle);
-
 error_exit:
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 	return (res);
 }
 
 int totempg_groups_leave (
-	hdb_handle_t handle,
+	void *totempg_groups_instance,
 	const struct totempg_group *groups,
 	size_t group_cnt)
 {
-	struct totempg_group_instance *instance;
-	unsigned int res;
-
-	pthread_mutex_lock (&totempg_mutex);
-	res = hdb_handle_get (&totempg_groups_instance_database, handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_exit;
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
 	}
 
-	hdb_handle_put (&totempg_groups_instance_database, handle);
-
-error_exit:
-	pthread_mutex_unlock (&totempg_mutex);
-	return (res);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
+	return (0);
 }
 
 #define MAX_IOVECS_FROM_APP 32
 #define MAX_GROUPS_PER_MSG 32
 
 int totempg_groups_mcast_joined (
-	hdb_handle_t handle,
+	void *totempg_groups_instance,
 	const struct iovec *iovec,
 	unsigned int iov_len,
 	int guarantee)
 {
-	struct totempg_group_instance *instance;
+	struct totempg_group_instance *instance = (struct totempg_group_instance *)totempg_groups_instance;
 	unsigned short group_len[MAX_GROUPS_PER_MSG + 1];
 	struct iovec iovec_mcast[MAX_GROUPS_PER_MSG + 1 + MAX_IOVECS_FROM_APP];
 	int i;
 	unsigned int res;
 
-	pthread_mutex_lock (&totempg_mutex);
-	res = hdb_handle_get (&totempg_groups_instance_database, handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_exit;
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
 	}
-
+	
 	/*
 	 * Build group_len structure and the iovec_mcast structure
 	 */
@@ -1266,30 +1253,56 @@ int totempg_groups_mcast_joined (
 	}
 
 	res = mcast_msg (iovec_mcast, iov_len + instance->groups_cnt + 1, guarantee);
-	hdb_handle_put (&totempg_groups_instance_database, handle);
 
-error_exit:
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
+	
 	return (res);
 }
 
+static void check_q_level(
+	void *totempg_groups_instance)
+{
+	struct totempg_group_instance *instance = (struct totempg_group_instance *)totempg_groups_instance;
+	int32_t old_level = instance->q_level;
+	int32_t percent_used = q_level_precent_used();
+
+	if (percent_used >= 75 && instance->q_level != TOTEM_Q_LEVEL_CRITICAL) {
+		instance->q_level = TOTEM_Q_LEVEL_CRITICAL;
+	} else if (percent_used < 30 && instance->q_level != TOTEM_Q_LEVEL_LOW) {
+		instance->q_level = TOTEM_Q_LEVEL_LOW;
+	} else if (percent_used > 40 && percent_used < 50 && instance->q_level != TOTEM_Q_LEVEL_GOOD) {
+		instance->q_level = TOTEM_Q_LEVEL_GOOD;
+	} else if (percent_used > 60 && percent_used < 70 && instance->q_level != TOTEM_Q_LEVEL_HIGH) {
+		instance->q_level = TOTEM_Q_LEVEL_HIGH;
+	}
+	if (totem_queue_level_changed && old_level != instance->q_level) {
+		totem_queue_level_changed(instance->q_level);
+	}
+}
+
+void totempg_check_q_level(
+	void *totempg_groups_instance)
+{
+	struct totempg_group_instance *instance = (struct totempg_group_instance *)totempg_groups_instance;
+
+	check_q_level(instance);
+}
+
 int totempg_groups_joined_reserve (
-	hdb_handle_t handle,
+	void *totempg_groups_instance,
 	const struct iovec *iovec,
 	unsigned int iov_len)
 {
-	struct totempg_group_instance *instance;
+	struct totempg_group_instance *instance = (struct totempg_group_instance *)totempg_groups_instance;
 	unsigned int size = 0;
 	unsigned int i;
-	unsigned int res;
 	unsigned int reserved = 0;
 
-	pthread_mutex_lock (&totempg_mutex);
-	pthread_mutex_lock (&mcast_msg_mutex);
-	res = hdb_handle_get (&totempg_groups_instance_database, handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_exit;
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
+		pthread_mutex_lock (&mcast_msg_mutex);
 	}
 
 	for (i = 0; i < instance->groups_cnt; i++) {
@@ -1298,56 +1311,58 @@ int totempg_groups_joined_reserve (
 	for (i = 0; i < iov_len; i++) {
 		size += iovec[i].iov_len;
 	}
+
 	if (size >= totempg_size_limit) {
 		reserved = -1;
-		goto error_put;
+		goto error_exit;
 	}
 
-	reserved = send_reserve (size);
-	if (msg_count_send_ok (reserved) == 0) {
-		send_release (reserved);
+	if (byte_count_send_ok (size)) {
+		reserved = send_reserve (size);
+	} else {
 		reserved = 0;
 	}
 
-error_put:
-	hdb_handle_put (&totempg_groups_instance_database, handle);
-
 error_exit:
-	pthread_mutex_unlock (&mcast_msg_mutex);
-	pthread_mutex_unlock (&totempg_mutex);
+	check_q_level(instance);
+
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&mcast_msg_mutex);
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 	return (reserved);
 }
 
 
 int totempg_groups_joined_release (int msg_count)
 {
-	pthread_mutex_lock (&totempg_mutex);
-	pthread_mutex_lock (&mcast_msg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
+		pthread_mutex_lock (&mcast_msg_mutex);
+	}
 	send_release (msg_count);
-	pthread_mutex_unlock (&mcast_msg_mutex);
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&mcast_msg_mutex);
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 	return 0;
 }
 
 int totempg_groups_mcast_groups (
-	hdb_handle_t handle,
+	void *totempg_groups_instance,
 	int guarantee,
 	const struct totempg_group *groups,
 	size_t groups_cnt,
 	const struct iovec *iovec,
 	unsigned int iov_len)
 {
-	struct totempg_group_instance *instance;
 	unsigned short group_len[MAX_GROUPS_PER_MSG + 1];
 	struct iovec iovec_mcast[MAX_GROUPS_PER_MSG + 1 + MAX_IOVECS_FROM_APP];
 	int i;
 	unsigned int res;
 
-	pthread_mutex_lock (&totempg_mutex);
-	res = hdb_handle_get (&totempg_groups_instance_database, handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_exit;
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
 	}
 
 	/*
@@ -1368,10 +1383,9 @@ int totempg_groups_mcast_groups (
 
 	res = mcast_msg (iovec_mcast, iov_len + groups_cnt + 1, guarantee);
 
-	hdb_handle_put (&totempg_groups_instance_database, handle);
-
-error_exit:
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 	return (res);
 }
 
@@ -1379,22 +1393,18 @@ error_exit:
  * Returns -1 if error, 0 if can't send, 1 if can send the message
  */
 int totempg_groups_send_ok_groups (
-	hdb_handle_t handle,
+	void *totempg_groups_instance,
 	const struct totempg_group *groups,
 	size_t groups_cnt,
 	const struct iovec *iovec,
 	unsigned int iov_len)
 {
-	struct totempg_group_instance *instance;
 	unsigned int size = 0;
 	unsigned int i;
 	unsigned int res;
 
-	pthread_mutex_lock (&totempg_mutex);
-	res = hdb_handle_get (&totempg_groups_instance_database, handle,
-		(void *)&instance);
-	if (res != 0) {
-		goto error_exit;
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_lock (&totempg_mutex);
 	}
 
 	for (i = 0; i < groups_cnt; i++) {
@@ -1406,15 +1416,16 @@ int totempg_groups_send_ok_groups (
 
 	res = msg_count_send_ok (size);
 
-	hdb_handle_put (&totempg_groups_instance_database, handle);
-error_exit:
-	pthread_mutex_unlock (&totempg_mutex);
+	if (totempg_threaded_mode == 1) {
+		pthread_mutex_unlock (&totempg_mutex);
+	}
 	return (res);
 }
 
 int totempg_ifaces_get (
 	unsigned int nodeid,
 	struct totem_ip_address *interfaces,
+	unsigned int interfaces_size,
 	char ***status,
 	unsigned int *iface_count)
 {
@@ -1423,6 +1434,7 @@ int totempg_ifaces_get (
 	res = totemmrp_ifaces_get (
 		nodeid,
 		interfaces,
+		interfaces_size,
 		status,
 		iface_count);
 
@@ -1440,12 +1452,12 @@ void* totempg_get_stats (void)
 }
 
 int totempg_crypto_set (
-	unsigned int type)
+	const char *cipher_type,
+	const char *hash_type)
 {
 	int res;
 
-	res = totemmrp_crypto_set (
-		type);
+	res = totemmrp_crypto_set (cipher_type, hash_type);
 
 	return (res);
 }
@@ -1459,10 +1471,11 @@ int totempg_ring_reenable (void)
 	return (res);
 }
 
+#define ONE_IFACE_LEN 63
 const char *totempg_ifaces_print (unsigned int nodeid)
 {
 	static char iface_string[256 * INTERFACE_MAX];
-	char one_iface[64];
+	char one_iface[ONE_IFACE_LEN+1];
 	struct totem_ip_address interfaces[INTERFACE_MAX];
 	char **status;
 	unsigned int iface_count;
@@ -1471,14 +1484,17 @@ const char *totempg_ifaces_print (unsigned int nodeid)
 
 	iface_string[0] = '\0';
 
-	res = totempg_ifaces_get (nodeid, interfaces, &status, &iface_count);
+	res = totempg_ifaces_get (nodeid, interfaces, INTERFACE_MAX, &status, &iface_count);
 	if (res == -1) {
 		return ("no interface found for nodeid");
 	}
 
+	res = totempg_ifaces_get (nodeid, interfaces, INTERFACE_MAX, &status, &iface_count);
+
 	for (i = 0; i < iface_count; i++) {
-		sprintf (one_iface, "r(%d) ip(%s) ",
-			i, totemip_print (&interfaces[i]));
+		snprintf (one_iface, ONE_IFACE_LEN,
+			  "r(%d) ip(%s) ",
+			  i, totemip_print (&interfaces[i]));
 		strcat (iface_string, one_iface);
 	}
 	return (iface_string);
@@ -1499,15 +1515,33 @@ extern void totempg_service_ready_register (
 	totemmrp_service_ready_register (totem_service_ready);
 }
 
+void totempg_queue_level_register_callback (totem_queue_level_changed_fn fn)
+{
+	totem_queue_level_changed = fn;
+}
+
 extern int totempg_member_add (
 	const struct totem_ip_address *member,
-	int ring_no);
+	int ring_no)
+{
+	return totemmrp_member_add (member, ring_no);
+}
 
 extern int totempg_member_remove (
 	const struct totem_ip_address *member,
-	int ring_no);
+	int ring_no)
+{
+	return totemmrp_member_remove (member, ring_no);
+}
+
+void totempg_threaded_mode_enable (void)
+{
+	totempg_threaded_mode = 1;
+	totemmrp_threaded_mode_enable ();
+}
 
 void totempg_trans_ack (void)
 {
 	totemmrp_trans_ack ();
 }
+
