@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2009 Red Hat, Inc.
+ * Copyright (c) 2006-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -55,26 +55,26 @@
 #include <time.h>
 #include <sys/time.h>
 #include <sys/poll.h>
+#include <sys/uio.h>
 #include <limits.h>
 
+#include <qb/qbdefs.h>
+#include <qb/qbloop.h>
+
 #include <corosync/sq.h>
 #include <corosync/list.h>
-#include <corosync/hdb.h>
 #include <corosync/swab.h>
-#include <corosync/totem/coropoll.h>
 #define LOGSYS_UTILS_ONLY 1
-#include <corosync/engine/logsys.h>
+#include <corosync/logsys.h>
 #include "totemudpu.h"
 
-#include "crypto.h"
 #include "util.h"
+#include "totemcrypto.h"
 
-#ifdef HAVE_LIBNSS
 #include <nss.h>
 #include <pk11pub.h>
 #include <pkcs11.h>
 #include <prerror.h>
-#endif
 
 #ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
@@ -88,34 +88,16 @@
 #define BIND_STATE_REGULAR	1
 #define BIND_STATE_LOOPBACK	2
 
-#define HMAC_HASH_SIZE 20
-struct security_header {
-	unsigned char hash_digest[HMAC_HASH_SIZE]; /* The hash *MUST* be first in the data structure */
-	unsigned char salt[16]; /* random number */
-	char msg[0];
-} __attribute__((packed));
-
 struct totemudpu_member {
 	struct list_head list;
 	struct totem_ip_address member;
 	int fd;
 };
-	
+
 struct totemudpu_instance {
-	hmac_state totemudpu_hmac_state;
+	struct crypto_instance *crypto_inst;
 
-	prng_state totemudpu_prng_state;
-
-#ifdef HAVE_LIBNSS
-	PK11SymKey   *nss_sym_key;
-	PK11SymKey   *nss_sym_key_sign;
-#endif
-
-	unsigned char totemudpu_private_key[1024];
-
-	unsigned int totemudpu_private_key_len;
-
-	hdb_handle_t totemudpu_poll_handle;
+	qb_loop_t *totemudpu_poll_handle;
 
 	struct totem_interface *totem_interface;
 
@@ -152,12 +134,13 @@ struct totemudpu_instance {
 	int totemudpu_subsys_id;
 
 	void (*totemudpu_log_printf) (
-		unsigned int rec_ident,
+		int level,
+		int subsys,
 		const char *function,
 		const char *file,
 		int line,
 		const char *format,
-		...)__attribute__((format(printf, 5, 6)));
+		...)__attribute__((format(printf, 6, 7)));
 
 	void *udpu_context;
 
@@ -183,12 +166,14 @@ struct totemudpu_instance {
 
 	int firstrun;
 
-	poll_timer_handle timer_netif_check_timeout;
+	qb_loop_timer_handle timer_netif_check_timeout;
 
 	unsigned int my_memb_entries;
 
 	struct totem_config *totem_config;
 
+	totemsrp_stats_t *stats;
+
 	struct totem_ip_address token_target;
 
 	int token_socket;
@@ -232,654 +217,30 @@ static void totemudpu_instance_initialize (struct totemudpu_instance *instance)
 	list_init (&instance->member_list);
 }
 
-#define log_printf(level, format, args...)				\
-do {									\
-        instance->totemudpu_log_printf (					\
-		LOGSYS_ENCODE_RECID(level,				\
-				    instance->totemudpu_subsys_id,	\
-				    LOGSYS_RECID_LOG),			\
-                __FUNCTION__, __FILE__, __LINE__,			\
-		(const char *)format, ##args);				\
+#define log_printf(level, format, args...)		\
+do {							\
+        instance->totemudpu_log_printf (		\
+		level, instance->totemudpu_subsys_id,	\
+                __FUNCTION__, __FILE__, __LINE__,	\
+		(const char *)format, ##args);		\
 } while (0);
-
-static int authenticate_and_decrypt_sober (
-	struct totemudpu_instance *instance,
-	struct iovec *iov,
-	unsigned int iov_len)
-{
-	unsigned char keys[48];
-	struct security_header *header = (struct security_header *)iov[0].iov_base;
-	prng_state keygen_prng_state;
-	prng_state stream_prng_state;
-	unsigned char *hmac_key = &keys[32];
-	unsigned char *cipher_key = &keys[16];
-	unsigned char *initial_vector = &keys[0];
-	unsigned char digest_comparison[HMAC_HASH_SIZE];
-	unsigned long len;
-
-	/*
-	 * Generate MAC, CIPHER, IV keys from private key
-	 */
-	memset (keys, 0, sizeof (keys));
-	sober128_start (&keygen_prng_state);
-	sober128_add_entropy (instance->totemudpu_private_key,
-		instance->totemudpu_private_key_len, &keygen_prng_state);
-	sober128_add_entropy (header->salt, sizeof (header->salt), &keygen_prng_state);
-
-	sober128_read (keys, sizeof (keys), &keygen_prng_state);
-
-	/*
-	 * Setup stream cipher
-	 */
-	sober128_start (&stream_prng_state);
-	sober128_add_entropy (cipher_key, 16, &stream_prng_state);
-	sober128_add_entropy (initial_vector, 16, &stream_prng_state);
-
-	/*
-	 * Authenticate contents of message
-	 */
-	hmac_init (&instance->totemudpu_hmac_state, DIGEST_SHA1, hmac_key, 16);
-
-	hmac_process (&instance->totemudpu_hmac_state,
-		(unsigned char *)iov->iov_base + HMAC_HASH_SIZE,
-		iov->iov_len - HMAC_HASH_SIZE);
-
-	len = hash_descriptor[DIGEST_SHA1]->hashsize;
-	assert (HMAC_HASH_SIZE >= len);
-	hmac_done (&instance->totemudpu_hmac_state, digest_comparison, &len);
-
-	if (memcmp (digest_comparison, header->hash_digest, len) != 0) {
-		return (-1);
-	}
-
-	/*
-	 * Decrypt the contents of the message with the cipher key
-	 */
-	sober128_read ((unsigned char*)iov->iov_base +
-			sizeof (struct security_header),
-		iov->iov_len - sizeof (struct security_header),
-		&stream_prng_state);
-
-	return (0);
-}
-
-static void init_sober_crypto(
-	struct totemudpu_instance *instance)
-{
-	log_printf(instance->totemudpu_log_level_notice,
-		"Initializing transmit/receive security: libtomcrypt SOBER128/SHA1HMAC (mode 0).\n");
-	rng_make_prng (128, PRNG_SOBER, &instance->totemudpu_prng_state, NULL);
-}
-
-#ifdef HAVE_LIBNSS
-
-static unsigned char *copy_from_iovec(
-	const struct iovec *iov,
-	unsigned int iov_len,
-	size_t *buf_size)
-{
-	int i;
-	size_t bufptr;
-	size_t buflen = 0;
-	unsigned char *newbuf;
-
-	for (i=0; i<iov_len; i++)
-		buflen += iov[i].iov_len;
-
-	newbuf = malloc(buflen);
-	if (!newbuf)
-		return NULL;
-
-	bufptr=0;
-	for (i=0; i<iov_len; i++) {
-		memcpy(newbuf+bufptr, iov[i].iov_base, iov[i].iov_len);
-		bufptr += iov[i].iov_len;
-	}
-	*buf_size = buflen;
-	return newbuf;
-}
-
-static void copy_to_iovec(
-	struct iovec *iov,
-	unsigned int iov_len,
-	const unsigned char *buf,
-	size_t buf_size)
-{
-	int i;
-	size_t copylen;
-	size_t bufptr = 0;
-
-	bufptr=0;
-	for (i=0; i<iov_len; i++) {
-		copylen = iov[i].iov_len;
-		if (bufptr + copylen > buf_size) {
-			copylen = buf_size - bufptr;
-		}
-		memcpy(iov[i].iov_base, buf+bufptr, copylen);
-		bufptr += copylen;
-		if (iov[i].iov_len != copylen) {
-			iov[i].iov_len = copylen;
-			return;
-		}
-	}
-}
-
-static void init_nss_crypto(
-	struct totemudpu_instance *instance)
-{
-	PK11SlotInfo*      aes_slot = NULL;
-	PK11SlotInfo*      sha1_slot = NULL;
-	SECItem            key_item;
-	SECStatus          rv;
-
-	log_printf(instance->totemudpu_log_level_notice,
-		"Initializing transmit/receive security: NSS AES128CBC/SHA1HMAC (mode 1).\n");
-	rv = NSS_NoDB_Init(".");
-	if (rv != SECSuccess)
-	{
-		log_printf(instance->totemudpu_log_level_security, "NSS initialization failed (err %d)\n",
-			PR_GetError());
-		goto out;
-	}
-
-	aes_slot = PK11_GetBestSlot(instance->totem_config->crypto_crypt_type, NULL);
-	if (aes_slot == NULL)
-	{
-		log_printf(instance->totemudpu_log_level_security, "Unable to find security slot (err %d)\n",
-			PR_GetError());
-		goto out;
-	}
-
-	sha1_slot = PK11_GetBestSlot(CKM_SHA_1_HMAC, NULL);
-	if (sha1_slot == NULL)
-	{
-		log_printf(instance->totemudpu_log_level_security, "Unable to find security slot (err %d)\n",
-			PR_GetError());
-		goto out;
-	}
-	/*
-	 * Make the private key into a SymKey that we can use
-	 */
-	key_item.type = siBuffer;
-	key_item.data = instance->totem_config->private_key;
-	key_item.len = 32; /* Use 128 bits */
-
-	instance->nss_sym_key = PK11_ImportSymKey(aes_slot,
-		instance->totem_config->crypto_crypt_type,
-		PK11_OriginUnwrap, CKA_ENCRYPT|CKA_DECRYPT,
-		&key_item, NULL);
-	if (instance->nss_sym_key == NULL)
-	{
-		log_printf(instance->totemudpu_log_level_security, "Failure to import key into NSS (err %d)\n",
-			PR_GetError());
-		goto out;
-	}
-
-	instance->nss_sym_key_sign = PK11_ImportSymKey(sha1_slot,
-		CKM_SHA_1_HMAC,
-		PK11_OriginUnwrap, CKA_SIGN,
-		&key_item, NULL);
-	if (instance->nss_sym_key_sign == NULL) {
-		log_printf(instance->totemudpu_log_level_security, "Failure to import key into NSS (err %d)\n",
-			PR_GetError());
-		goto out;
-	}
-out:
-	return;
-}
-
-static int encrypt_and_sign_nss (
-	struct totemudpu_instance *instance,
-	unsigned char *buf,
-	size_t *buf_len,
-	const struct iovec *iovec,
-	unsigned int iov_len)
-{
-	PK11Context*       enc_context = NULL;
-	SECStatus          rv1, rv2;
-	int                tmp1_outlen;
-	unsigned int       tmp2_outlen;
-	unsigned char      *inbuf;
-	unsigned char      *data;
-	unsigned char      *outdata;
-	size_t             datalen;
-	SECItem            no_params;
-	SECItem            iv_item;
-	struct security_header *header;
-	SECItem      *nss_sec_param;
-	unsigned char nss_iv_data[16];
-	SECStatus          rv;
-
-	no_params.type = siBuffer;
-	no_params.data = 0;
-	no_params.len = 0;
-
-	tmp1_outlen = tmp2_outlen = 0;
-	inbuf = copy_from_iovec(iovec, iov_len, &datalen);
-	if (!inbuf) {
-		log_printf(instance->totemudpu_log_level_security, "malloc error copying buffer from iovec\n");
-		goto out;
-	}
-
-	data = inbuf + sizeof (struct security_header);
-	datalen -= sizeof (struct security_header);
-
-	outdata = buf + sizeof (struct security_header);
-	header = (struct security_header *)buf;
-
-	rv = PK11_GenerateRandom (
-		nss_iv_data,
-		sizeof (nss_iv_data));
-	if (rv != SECSuccess) {
-		log_printf(instance->totemudpu_log_level_security,
-			"Failure to generate a random number %d\n",
-			PR_GetError());
-		goto out;
-	}
-
-	memcpy(header->salt, nss_iv_data, sizeof(nss_iv_data));
-	iv_item.type = siBuffer;
-	iv_item.data = nss_iv_data;
-	iv_item.len = sizeof (nss_iv_data);
-
-	nss_sec_param = PK11_ParamFromIV (
-		instance->totem_config->crypto_crypt_type,
-		&iv_item);
-	if (nss_sec_param == NULL) {
-		log_printf(instance->totemudpu_log_level_security,
-			"Failure to set up PKCS11 param (err %d)\n",
-			PR_GetError());
-		free (inbuf);
-		goto out;
-	}
-
-	/*
-	 * Create cipher context for encryption
-	 */
-	enc_context = PK11_CreateContextBySymKey (
-		instance->totem_config->crypto_crypt_type,
-		CKA_ENCRYPT,
-		instance->nss_sym_key,
-		nss_sec_param);
-	if (!enc_context) {
-		char err[1024];
-		PR_GetErrorText(err);
-		err[PR_GetErrorTextLength()] = 0;
-		log_printf(instance->totemudpu_log_level_security,
-			"PK11_CreateContext failed (encrypt) crypt_type=%d (err %d): %s\n",
-			instance->totem_config->crypto_crypt_type,
-			PR_GetError(), err);
-		free(inbuf);
-		goto sec_out;
-	}
-	rv1 = PK11_CipherOp(enc_context, outdata,
-			    &tmp1_outlen, FRAME_SIZE_MAX - sizeof(struct security_header),
-			    data, datalen);
-	rv2 = PK11_DigestFinal(enc_context, outdata + tmp1_outlen, &tmp2_outlen,
-			       FRAME_SIZE_MAX - tmp1_outlen);
-	PK11_DestroyContext(enc_context, PR_TRUE);
-
-	*buf_len = tmp1_outlen + tmp2_outlen;
-	free(inbuf);
-//	memcpy(&outdata[*buf_len], nss_iv_data, sizeof(nss_iv_data));
-
-	if (rv1 != SECSuccess || rv2 != SECSuccess)
-		goto sec_out;
-
-	/* Now do the digest */
-	enc_context = PK11_CreateContextBySymKey(CKM_SHA_1_HMAC,
-		CKA_SIGN, instance->nss_sym_key_sign, &no_params);
-	if (!enc_context) {
-		char err[1024];
-		PR_GetErrorText(err);
-		err[PR_GetErrorTextLength()] = 0;
-		log_printf(instance->totemudpu_log_level_security, "encrypt: PK11_CreateContext failed (digest) err %d: %s\n",
-			PR_GetError(), err);
-		goto sec_out;
-	}
-
-
-	PK11_DigestBegin(enc_context);
-
-	rv1 = PK11_DigestOp(enc_context, outdata - 16, *buf_len + 16);
-	rv2 = PK11_DigestFinal(enc_context, header->hash_digest, &tmp2_outlen, sizeof(header->hash_digest));
-
-	PK11_DestroyContext(enc_context, PR_TRUE);
-
-	if (rv1 != SECSuccess || rv2 != SECSuccess)
-		goto sec_out;
-
-
-	*buf_len = *buf_len + sizeof(struct security_header);
-	SECITEM_FreeItem(nss_sec_param, PR_TRUE);
-	return 0;
-
-sec_out:
-	if (nss_sec_param != NULL) {
-		SECITEM_FreeItem(nss_sec_param, PR_TRUE);
-	}
-out:
-	return -1;
-}
-
-
-static int authenticate_and_decrypt_nss (
-	struct totemudpu_instance *instance,
-	struct iovec *iov,
-	unsigned int iov_len)
-{
-	PK11Context*  enc_context = NULL;
-	SECStatus     rv1, rv2;
-	int           tmp1_outlen;
-	unsigned int  tmp2_outlen;
-	unsigned char outbuf[FRAME_SIZE_MAX];
-	unsigned char digest[HMAC_HASH_SIZE];
-	unsigned char *outdata;
-	int           result_len;
-	unsigned char *data;
-	unsigned char *inbuf;
-	size_t        datalen;
-	struct security_header *header = (struct security_header *)iov[0].iov_base;
-	SECItem no_params;
-	SECItem ivdata;
-
-	no_params.type = siBuffer;
-	no_params.data = 0;
-	no_params.len = 0;
-
-	tmp1_outlen = tmp2_outlen = 0;
-	if (iov_len > 1) {
-		inbuf = copy_from_iovec(iov, iov_len, &datalen);
-		if (!inbuf) {
-			log_printf(instance->totemudpu_log_level_security, "malloc error copying buffer from iovec\n");
-			return -1;
-		}
-	}
-	else {
-		inbuf = (unsigned char *)iov[0].iov_base;
-		datalen = iov[0].iov_len;
-	}
-	data = inbuf + sizeof (struct security_header) - 16;
-	datalen = datalen - sizeof (struct security_header) + 16;
-
-	outdata = outbuf + sizeof (struct security_header);
-
-	/* Check the digest */
-	enc_context = PK11_CreateContextBySymKey (
-		CKM_SHA_1_HMAC, CKA_SIGN,
-		instance->nss_sym_key_sign,
-		&no_params);
-	if (!enc_context) {
-		char err[1024];
-		PR_GetErrorText(err);
-		err[PR_GetErrorTextLength()] = 0;
-		log_printf(instance->totemudpu_log_level_security, "PK11_CreateContext failed (check digest) err %d: %s\n",
-			PR_GetError(), err);
-		goto out;
-	}
-
-	PK11_DigestBegin(enc_context);
-
-	rv1 = PK11_DigestOp(enc_context, data, datalen);
-	rv2 = PK11_DigestFinal(enc_context, digest, &tmp2_outlen, sizeof(digest));
-
-	PK11_DestroyContext(enc_context, PR_TRUE);
-
-	if (rv1 != SECSuccess || rv2 != SECSuccess) {
-		log_printf(instance->totemudpu_log_level_security, "Digest check failed\n");
-		goto out;
-	}
-
-	if (memcmp(digest, header->hash_digest, tmp2_outlen) != 0) {
-		log_printf(instance->totemudpu_log_level_error, "Digest does not match\n");
-		goto out;
-	}
-
-	/*
-	 * Get rid of salt
-	 */
-	data += 16;
-	datalen -= 16;
-
-	/* Create cipher context for decryption */
-	ivdata.type = siBuffer;
-	ivdata.data = header->salt;
-	ivdata.len = sizeof(header->salt);
-
-	enc_context = PK11_CreateContextBySymKey(
-		instance->totem_config->crypto_crypt_type,
-		CKA_DECRYPT,
-		instance->nss_sym_key, &ivdata);
-	if (!enc_context) {
-		log_printf(instance->totemudpu_log_level_security,
-			"PK11_CreateContext (decrypt) failed (err %d)\n",
-			PR_GetError());
-		goto out;
-	}
-
-	rv1 = PK11_CipherOp(enc_context, outdata, &tmp1_outlen,
-			    sizeof(outbuf) - sizeof (struct security_header),
-			    data, datalen);
-	if (rv1 != SECSuccess) {
-		log_printf(instance->totemudpu_log_level_security,
-			"PK11_CipherOp (decrypt) failed (err %d)\n",
-			PR_GetError());
-	}
-	rv2 = PK11_DigestFinal(enc_context, outdata + tmp1_outlen, &tmp2_outlen,
-			       sizeof(outbuf) - tmp1_outlen);
-	PK11_DestroyContext(enc_context, PR_TRUE);
-	result_len = tmp1_outlen + tmp2_outlen + sizeof (struct security_header);
-
-	/* Copy it back to the buffer */
-	copy_to_iovec(iov, iov_len, outbuf, result_len);
-	if (iov_len > 1)
-		free(inbuf);
-
-	if (rv1 != SECSuccess || rv2 != SECSuccess)
-		return -1;
-
-	return 0;
-
-out:
-	if (iov_len > 1 && inbuf != NULL) {
-		free (inbuf);
-	}
-
-	return (-1);
-}
-#endif
-
-static int encrypt_and_sign_sober (
-	struct totemudpu_instance *instance,
-	unsigned char *buf,
-	size_t *buf_len,
-	const struct iovec *iovec,
-	unsigned int iov_len)
-{
-	int i;
-	unsigned char *addr;
-	unsigned char keys[48];
-	struct security_header *header;
-	unsigned char *hmac_key = &keys[32];
-	unsigned char *cipher_key = &keys[16];
-	unsigned char *initial_vector = &keys[0];
-	unsigned long len;
-	size_t outlen = 0;
-	hmac_state hmac_st;
-	prng_state keygen_prng_state;
-	prng_state stream_prng_state;
-	prng_state *prng_state_in = &instance->totemudpu_prng_state;
-
-	header = (struct security_header *)buf;
-	addr = buf + sizeof (struct security_header);
-
-	memset (keys, 0, sizeof (keys));
-	memset (header->salt, 0, sizeof (header->salt));
-
-	/*
-	 * Generate MAC, CIPHER, IV keys from private key
-	 */
-	sober128_read (header->salt, sizeof (header->salt), prng_state_in);
-	sober128_start (&keygen_prng_state);
-	sober128_add_entropy (instance->totemudpu_private_key,
-		instance->totemudpu_private_key_len,
-		&keygen_prng_state);
-	sober128_add_entropy (header->salt, sizeof (header->salt),
-		&keygen_prng_state);
-
-	sober128_read (keys, sizeof (keys), &keygen_prng_state);
-
-	/*
-	 * Setup stream cipher
-	 */
-	sober128_start (&stream_prng_state);
-	sober128_add_entropy (cipher_key, 16, &stream_prng_state);
-	sober128_add_entropy (initial_vector, 16, &stream_prng_state);
-
-	outlen = sizeof (struct security_header);
-	/*
-	 * Copy remainder of message, then encrypt it
-	 */
-	for (i = 1; i < iov_len; i++) {
-		memcpy (addr, iovec[i].iov_base, iovec[i].iov_len);
-		addr += iovec[i].iov_len;
-		outlen += iovec[i].iov_len;
-	}
-
-	/*
- 	 * Encrypt message by XORing stream cipher data
-	 */
-	sober128_read (buf + sizeof (struct security_header),
-		outlen - sizeof (struct security_header),
-		&stream_prng_state);
-
-	memset (&hmac_st, 0, sizeof (hmac_st));
-
-	/*
-	 * Sign the contents of the message with the hmac key and store signature in message
-	 */
-	hmac_init (&hmac_st, DIGEST_SHA1, hmac_key, 16);
-
-	hmac_process (&hmac_st,
-		buf + HMAC_HASH_SIZE,
-		outlen - HMAC_HASH_SIZE);
-
-	len = hash_descriptor[DIGEST_SHA1]->hashsize;
-
-	hmac_done (&hmac_st, header->hash_digest, &len);
-
-	*buf_len = outlen;
-
-	return 0;
-}
-
-static int encrypt_and_sign_worker (
-	struct totemudpu_instance *instance,
-	unsigned char *buf,
-	size_t *buf_len,
-	const struct iovec *iovec,
-	unsigned int iov_len)
-{
-	if (instance->totem_config->crypto_type == TOTEM_CRYPTO_SOBER ||
-	    instance->totem_config->crypto_accept == TOTEM_CRYPTO_ACCEPT_OLD)
-		return encrypt_and_sign_sober(instance, buf, buf_len, iovec, iov_len);
-#ifdef HAVE_LIBNSS
-	if (instance->totem_config->crypto_type == TOTEM_CRYPTO_NSS)
-		return encrypt_and_sign_nss(instance, buf, buf_len, iovec, iov_len);
-#endif
-	return -1;
-}
-
-static int authenticate_and_decrypt (
-	struct totemudpu_instance *instance,
-	struct iovec *iov,
-	unsigned int iov_len)
-{
-	unsigned char type;
-	unsigned char *endbuf = (unsigned char *)iov[iov_len-1].iov_base;
-	int res = -1;
-
-	/*
-	 * Get the encryption type and remove it from the buffer
-	 */
-	type = endbuf[iov[iov_len-1].iov_len-1];
-	iov[iov_len-1].iov_len -= 1;
-
-	if (type == TOTEM_CRYPTO_SOBER)
-		res = authenticate_and_decrypt_sober(instance, iov, iov_len);
-
-	/*
-	 * Only try higher crypto options if NEW has been requested
-	 */
-	if (instance->totem_config->crypto_accept == TOTEM_CRYPTO_ACCEPT_NEW) {
-#ifdef HAVE_LIBNSS
-		if (type == TOTEM_CRYPTO_NSS)
-		    res = authenticate_and_decrypt_nss(instance, iov, iov_len);
-#endif
-	}
-
-	/*
-	 * If it failed, then try decrypting the whole packet as it might be
-	 * from aisexec
-	 */
-	if (res == -1) {
-		iov[iov_len-1].iov_len += 1;
-		res = authenticate_and_decrypt_sober(instance, iov, iov_len);
-	}
-
-	return res;
-}
-
-static void init_crypto(
-	struct totemudpu_instance *instance)
-{
-	/*
-	 * If we are expecting NEW crypto type then initialise all available
-	 * crypto options. For OLD then we only need SOBER128.
-	 */
-
-	init_sober_crypto(instance);
-
-	if (instance->totem_config->crypto_accept == TOTEM_CRYPTO_ACCEPT_OLD)
-		return;
-
-#ifdef HAVE_LIBNSS
-	init_nss_crypto(instance);
-#endif
-}
+#define LOGSYS_PERROR(err_num, level, fmt, args...)						\
+do {												\
+	char _error_str[LOGSYS_MAX_PERROR_MSG_LEN];						\
+	const char *_error_ptr = qb_strerror_r(err_num, _error_str, sizeof(_error_str));	\
+        instance->totemudpu_log_printf (							\
+		level, instance->totemudpu_subsys_id,						\
+                __FUNCTION__, __FILE__, __LINE__,						\
+		fmt ": %s (%d)", ##args, _error_ptr, err_num);				\
+	} while(0)
 
 int totemudpu_crypto_set (
 	void *udpu_context,
-	 unsigned int type)
+	const char *cipher_type,
+	const char *hash_type)
 {
-	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
-	int res = 0;
 
-	/*
-	 * Can't set crypto type if OLD is selected
-	 */
-	if (instance->totem_config->crypto_accept == TOTEM_CRYPTO_ACCEPT_OLD) {
-		res = -1;
-	} else {
-		/*
-		 * Validate crypto algorithm
-		 */
-		switch (type) {
-			case TOTEM_CRYPTO_SOBER:
-				log_printf(instance->totemudpu_log_level_security,
-					"Transmit security set to: libtomcrypt SOBER128/SHA1HMAC (mode 0)");
-				break;
-			case TOTEM_CRYPTO_NSS:
-				log_printf(instance->totemudpu_log_level_security,
-					"Transmit security set to: NSS AES128CBC/SHA1HMAC (mode 1)");
-				break;
-			default:
-				res = -1;
-				break;
-		}
-	}
-
-	return (res);
+	return (0);
 }
 
 
@@ -891,65 +252,51 @@ static inline void ucast_sendmsg (
 {
 	struct msghdr msg_ucast;
 	int res = 0;
-	size_t buf_len;
-	unsigned char sheader[sizeof (struct security_header)];
-	unsigned char encrypt_data[FRAME_SIZE_MAX];
-	struct iovec iovec_encrypt[2];
-	const struct iovec *iovec_sendmsg;
+	size_t buf_out_len;
+	unsigned char buf_out[FRAME_SIZE_MAX];
 	struct sockaddr_storage sockaddr;
 	struct iovec iovec;
-	unsigned int iov_len;
 	int addrlen;
 
-	if (instance->totem_config->secauth == 1) {
-		iovec_encrypt[0].iov_base = (void *)sheader;
-		iovec_encrypt[0].iov_len = sizeof (struct security_header);
-		iovec_encrypt[1].iov_base = (void *)msg;
-		iovec_encrypt[1].iov_len = msg_len;
-
-		/*
-		 * Encrypt and digest the message
-		 */
-		encrypt_and_sign_worker (
-			instance,
-			encrypt_data,
-			&buf_len,
-			iovec_encrypt,
-			2);
-
-		if (instance->totem_config->crypto_accept == TOTEM_CRYPTO_ACCEPT_NEW) {
-			encrypt_data[buf_len++] = instance->totem_config->crypto_type;
-		}
-		else {
-			encrypt_data[buf_len++] = 0;
-		}
-
-		iovec_encrypt[0].iov_base = (void *)encrypt_data;
-		iovec_encrypt[0].iov_len = buf_len;
-		iovec_sendmsg = &iovec_encrypt[0];
-		iov_len = 1;
-	} else {
-		iovec.iov_base = (void *)msg;
-		iovec.iov_len = msg_len;
-		iovec_sendmsg = &iovec;
-		iov_len = 1;
+	/*
+	 * Encrypt and digest the message
+	 */
+	if (crypto_encrypt_and_sign (
+		instance->crypto_inst,
+		(const unsigned char *)msg,
+		msg_len,
+		buf_out,
+		&buf_out_len) != 0) {
+		log_printf(LOGSYS_LEVEL_CRIT, "Error encrypting/signing packet (non-critical)");
+		return;
 	}
 
+	iovec.iov_base = (void *)buf_out;
+	iovec.iov_len = buf_out_len;
+
 	/*
 	 * Build unicast message
 	 */
 	totemip_totemip_to_sockaddr_convert(system_to,
 		instance->totem_interface->ip_port, &sockaddr, &addrlen);
+	memset(&msg_ucast, 0, sizeof(msg_ucast));
 	msg_ucast.msg_name = &sockaddr;
 	msg_ucast.msg_namelen = addrlen;
-	msg_ucast.msg_iov = (void *) iovec_sendmsg;
-	msg_ucast.msg_iovlen = iov_len;
-#if !defined(COROSYNC_SOLARIS)
+	msg_ucast.msg_iov = (void *)&iovec;
+	msg_ucast.msg_iovlen = 1;
+#ifdef HAVE_MSGHDR_CONTROL
 	msg_ucast.msg_control = 0;
+#endif
+#ifdef HAVE_MSGHDR_CONTROLLEN
 	msg_ucast.msg_controllen = 0;
+#endif
+#ifdef HAVE_MSGHDR_FLAGS
 	msg_ucast.msg_flags = 0;
-#else
+#endif
+#ifdef HAVE_MSGHDR_ACCRIGHTS
 	msg_ucast.msg_accrights = NULL;
+#endif
+#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
 	msg_ucast.msg_accrightslen = 0;
 #endif
 
@@ -972,53 +319,31 @@ static inline void mcast_sendmsg (
 {
 	struct msghdr msg_mcast;
 	int res = 0;
-	size_t buf_len;
-	unsigned char sheader[sizeof (struct security_header)];
-	unsigned char encrypt_data[FRAME_SIZE_MAX];
-	struct iovec iovec_encrypt[2];
+	size_t buf_out_len;
+	unsigned char buf_out[FRAME_SIZE_MAX];
 	struct iovec iovec;
-	const struct iovec *iovec_sendmsg;
 	struct sockaddr_storage sockaddr;
-	unsigned int iov_len;
 	int addrlen;
         struct list_head *list;
 	struct totemudpu_member *member;
 
-	if (instance->totem_config->secauth == 1) {
-		iovec_encrypt[0].iov_base = (void *)sheader;
-		iovec_encrypt[0].iov_len = sizeof (struct security_header);
-		iovec_encrypt[1].iov_base = (void *)msg;
-		iovec_encrypt[1].iov_len = msg_len;
-
-		/*
-		 * Encrypt and digest the message
-		 */
-		encrypt_and_sign_worker (
-			instance,
-			encrypt_data,
-			&buf_len,
-			iovec_encrypt,
-			2);
-
-		if (instance->totem_config->crypto_accept == TOTEM_CRYPTO_ACCEPT_NEW) {
-			encrypt_data[buf_len++] = instance->totem_config->crypto_type;
-		}
-		else {
-			encrypt_data[buf_len++] = 0;
-		}
-
-		iovec_encrypt[0].iov_base = (void *)encrypt_data;
-		iovec_encrypt[0].iov_len = buf_len;
-		iovec_sendmsg = &iovec_encrypt[0];
-		iov_len = 1;
-	} else {
-		iovec.iov_base = (void *)msg;
-		iovec.iov_len = msg_len;
-
-		iovec_sendmsg = &iovec;
-		iov_len = 1;
+	/*
+	 * Encrypt and digest the message
+	 */
+	if (crypto_encrypt_and_sign (
+		instance->crypto_inst,
+		(const unsigned char *)msg,
+		msg_len,
+		buf_out,
+		&buf_out_len) != 0) {
+		log_printf(LOGSYS_LEVEL_CRIT, "Error encrypting/signing packet (non-critical)");
+		return;
 	}
 
+	iovec.iov_base = (void *)buf_out;
+	iovec.iov_len = buf_out_len;
+
+	memset(&msg_mcast, 0, sizeof(msg_mcast));
 	/*
 	 * Build multicast message
 	 */
@@ -1034,14 +359,21 @@ static inline void mcast_sendmsg (
 			instance->totem_interface->ip_port, &sockaddr, &addrlen);
 		msg_mcast.msg_name = &sockaddr;
 		msg_mcast.msg_namelen = addrlen;
-		msg_mcast.msg_iov = (void *) iovec_sendmsg;
-		msg_mcast.msg_iovlen = iov_len;
-	#if !defined(COROSYNC_SOLARIS)
+		msg_mcast.msg_iov = (void *)&iovec;
+		msg_mcast.msg_iovlen = 1;
+	#ifdef HAVE_MSGHDR_CONTROL
 		msg_mcast.msg_control = 0;
+	#endif
+	#ifdef HAVE_MSGHDR_CONTROLLEN
 		msg_mcast.msg_controllen = 0;
+	#endif
+	#ifdef HAVE_MSGHDR_FLAGS
 		msg_mcast.msg_flags = 0;
-	#else
+	#endif
+	#ifdef HAVE_MSGHDR_ACCRIGHTS
 		msg_mcast.msg_accrights = NULL;
+	#endif
+	#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
 		msg_mcast.msg_accrightslen = 0;
 	#endif
 
@@ -1064,16 +396,15 @@ int totemudpu_finalize (
 	int res = 0;
 
 	if (instance->token_socket > 0) {
-		close (instance->token_socket);
-		poll_dispatch_delete (instance->totemudpu_poll_handle,
+		qb_loop_poll_del (instance->totemudpu_poll_handle,
 			instance->token_socket);
+		close (instance->token_socket);
 	}
 
 	return (res);
 }
 
 static int net_deliver_fn (
-	hdb_handle_t handle,
 	int fd,
 	int revents,
 	void *data)
@@ -1081,12 +412,9 @@ static int net_deliver_fn (
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)data;
 	struct msghdr msg_recv;
 	struct iovec *iovec;
-	struct security_header *security_header;
 	struct sockaddr_storage system_from;
 	int bytes_received;
 	int res = 0;
-	unsigned char *msg_offset;
-	unsigned int size_delv;
 
 	iovec = &instance->totemudpu_iov_recv;
 
@@ -1097,12 +425,19 @@ static int net_deliver_fn (
 	msg_recv.msg_namelen = sizeof (struct sockaddr_storage);
 	msg_recv.msg_iov = iovec;
 	msg_recv.msg_iovlen = 1;
-#if !defined(COROSYNC_SOLARIS)
+#ifdef HAVE_MSGHDR_CONTROL
 	msg_recv.msg_control = 0;
+#endif
+#ifdef HAVE_MSGHDR_CONTROLLEN
 	msg_recv.msg_controllen = 0;
+#endif
+#ifdef HAVE_MSGHDR_FLAGS
 	msg_recv.msg_flags = 0;
-#else
+#endif
+#ifdef HAVE_MSGHDR_ACCRIGHTS
 	msg_recv.msg_accrights = NULL;
+#endif
+#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
 	msg_recv.msg_accrightslen = 0;
 #endif
 
@@ -1113,44 +448,27 @@ static int net_deliver_fn (
 		instance->stats_recv += bytes_received;
 	}
 
-	if ((instance->totem_config->secauth == 1) &&
-		(bytes_received < sizeof (struct security_header))) {
+	/*
+	 * Authenticate and if authenticated, decrypt datagram
+	 */
 
-		log_printf (instance->totemudpu_log_level_security, "Received message is too short...  ignoring %d.\n", bytes_received);
-		return (0);
+	res = crypto_authenticate_and_decrypt (instance->crypto_inst, iovec->iov_base, &bytes_received);
+	if (res == -1) {
+		log_printf (instance->totemudpu_log_level_security, "Received message has invalid digest... ignoring.");
+		log_printf (instance->totemudpu_log_level_security,
+			"Invalid packet data");
+		iovec->iov_len = FRAME_SIZE_MAX;
+		return 0;
 	}
-
-	security_header = (struct security_header *)iovec->iov_base;
-
 	iovec->iov_len = bytes_received;
-	if (instance->totem_config->secauth == 1) {
-		/*
-		 * Authenticate and if authenticated, decrypt datagram
-		 */
-
-		res = authenticate_and_decrypt (instance, iovec, 1);
-		if (res == -1) {
-			log_printf (instance->totemudpu_log_level_security, "Received message has invalid digest... ignoring.\n");
-			log_printf (instance->totemudpu_log_level_security,
-				"Invalid packet data\n");
-			iovec->iov_len = FRAME_SIZE_MAX;
-			return 0;
-		}
-		msg_offset = (unsigned char *)iovec->iov_base +
-			sizeof (struct security_header);
-		size_delv = bytes_received - sizeof (struct security_header);
-	} else {
-		msg_offset = (void *)iovec->iov_base;
-		size_delv = bytes_received;
-	}
 
 	/*
 	 * Handle incoming message
 	 */
 	instance->totemudpu_deliver_fn (
 		instance->context,
-		msg_offset,
-		size_delv);
+		iovec->iov_base,
+		iovec->iov_len);
 
 	iovec->iov_len = FRAME_SIZE_MAX;
 	return (0);
@@ -1182,7 +500,6 @@ static void timer_function_netif_check_timeout (
 	void *data)
 {
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)data;
-	int res;
 	int interface_up;
 	int interface_num;
 	struct totem_ip_address *bind_address;
@@ -1205,8 +522,9 @@ static void timer_function_netif_check_timeout (
 		instance->netif_bind_state == BIND_STATE_REGULAR &&
 		interface_up == 1)) {
 
-		poll_timer_add (instance->totemudpu_poll_handle,
-			instance->totem_config->downcheck_timeout,
+		qb_loop_timer_add (instance->totemudpu_poll_handle,
+			QB_LOOP_MED,
+			instance->totem_config->downcheck_timeout*QB_TIME_NS_IN_MSEC,
 			(void *)instance,
 			timer_function_netif_check_timeout,
 			&instance->timer_netif_check_timeout);
@@ -1218,9 +536,9 @@ static void timer_function_netif_check_timeout (
 	}
 
 	if (instance->token_socket > 0) {
-		close (instance->token_socket);
-		poll_dispatch_delete (instance->totemudpu_poll_handle,
+		qb_loop_poll_del (instance->totemudpu_poll_handle,
 			instance->token_socket);
+		close (instance->token_socket);
 	}
 
 	if (interface_up == 0) {
@@ -1233,8 +551,9 @@ static void timer_function_netif_check_timeout (
 		/*
 		 * Add a timer to retry building interfaces and request memb_gather_enter
 		 */
-		poll_timer_add (instance->totemudpu_poll_handle,
-			instance->totem_config->downcheck_timeout,
+		qb_loop_timer_add (instance->totemudpu_poll_handle,
+			QB_LOOP_MED,
+			instance->totem_config->downcheck_timeout*QB_TIME_NS_IN_MSEC,
 			(void *)instance,
 			timer_function_netif_check_timeout,
 			&instance->timer_netif_check_timeout);
@@ -1248,12 +567,12 @@ static void timer_function_netif_check_timeout (
 	/*
 	 * Create and bind the multicast and unicast sockets
 	 */
-	res = totemudpu_build_sockets (instance,
+	totemudpu_build_sockets (instance,
 		bind_address,
 		&instance->totem_interface->boundto);
 
-	poll_dispatch_add (
-		instance->totemudpu_poll_handle,
+	qb_loop_poll_add (instance->totemudpu_poll_handle,
+		QB_LOOP_MED,
 		instance->token_socket,
 		POLLIN, instance, net_deliver_fn);
 
@@ -1265,7 +584,7 @@ static void timer_function_netif_check_timeout (
 	if (instance->netif_bind_state == BIND_STATE_REGULAR) {
 		if (instance->netif_state_report & NETIF_STATE_REPORT_UP) {
 			log_printf (instance->totemudpu_log_level_notice,
-				"The network interface [%s] is now up.\n",
+				"The network interface [%s] is now up.",
 				totemip_print (&instance->totem_interface->boundto));
 			instance->netif_state_report = NETIF_STATE_REPORT_DOWN;
 			instance->totemudpu_iface_change_fn (instance->context, &instance->my_id);
@@ -1274,8 +593,9 @@ static void timer_function_netif_check_timeout (
 		 * Add a timer to check for interface going down in single membership
 		 */
 		if (instance->my_memb_entries == 1) {
-			poll_timer_add (instance->totemudpu_poll_handle,
-				instance->totem_config->downcheck_timeout,
+			qb_loop_timer_add (instance->totemudpu_poll_handle,
+				QB_LOOP_MED,
+				instance->totem_config->downcheck_timeout*QB_TIME_NS_IN_MSEC,
 				(void *)instance,
 				timer_function_netif_check_timeout,
 				&instance->timer_netif_check_timeout);
@@ -1284,7 +604,7 @@ static void timer_function_netif_check_timeout (
 	} else {
 		if (instance->netif_state_report & NETIF_STATE_REPORT_DOWN) {
 			log_printf (instance->totemudpu_log_level_notice,
-				"The network interface is down.\n");
+				"The network interface is down.");
 			instance->totemudpu_iface_change_fn (instance->context, &instance->my_id);
 		}
 		instance->netif_state_report = NETIF_STATE_REPORT_UP;
@@ -1410,9 +730,10 @@ static int totemudpu_build_sockets (
  * Create an instance
  */
 int totemudpu_initialize (
-	hdb_handle_t poll_handle,
+	qb_loop_t *poll_handle,
 	void **udpu_context,
 	struct totem_config *totem_config,
+	totemsrp_stats_t *stats,
 	int interface_no,
 	void *context,
 
@@ -1438,6 +759,8 @@ int totemudpu_initialize (
 	totemudpu_instance_initialize (instance);
 
 	instance->totem_config = totem_config;
+	instance->stats = stats;
+
 	/*
 	* Configure logging
 	*/
@@ -1452,13 +775,18 @@ int totemudpu_initialize (
 	/*
 	* Initialize random number generator for later use to generate salt
 	*/
-	memcpy (instance->totemudpu_private_key, totem_config->private_key,
-		totem_config->private_key_len);
-
-	instance->totemudpu_private_key_len = totem_config->private_key_len;
-
-	init_crypto(instance);
-
+	instance->crypto_inst = crypto_init (totem_config->private_key,
+		totem_config->private_key_len,
+		totem_config->crypto_cipher_type,
+		totem_config->crypto_hash_type,
+		instance->totemudpu_log_printf,
+		instance->totemudpu_log_level_security,
+		instance->totemudpu_log_level_notice,
+		instance->totemudpu_log_level_error,
+		instance->totemudpu_subsys_id);
+	if (instance->crypto_inst == NULL) {
+		return (-1);
+	}
 	/*
 	 * Initialize local variables for totemudpu
 	 */
@@ -1483,8 +811,9 @@ int totemudpu_initialize (
 	 * RRP layer isn't ready to receive message because it hasn't
 	 * initialized yet.  Add short timer to check the interfaces.
 	 */
-	poll_timer_add (instance->totemudpu_poll_handle,
-		100,
+	qb_loop_timer_add (instance->totemudpu_poll_handle,
+		QB_LOOP_MED,
+		100*QB_TIME_NS_IN_MSEC,
 		(void *)instance,
 		timer_function_netif_check_timeout,
 		&instance->timer_netif_check_timeout);
@@ -1493,6 +822,16 @@ int totemudpu_initialize (
 	return (0);
 }
 
+void *totemudpu_buffer_alloc (void)
+{
+	return malloc (FRAME_SIZE_MAX);
+}
+
+void totemudpu_buffer_release (void *ptr)
+{
+	return free (ptr);
+}
+
 int totemudpu_processor_count_set (
 	void *udpu_context,
 	int processor_count)
@@ -1501,11 +840,12 @@ int totemudpu_processor_count_set (
 	int res = 0;
 
 	instance->my_memb_entries = processor_count;
-	poll_timer_delete (instance->totemudpu_poll_handle,
+	qb_loop_timer_del (instance->totemudpu_poll_handle,
 		instance->timer_netif_check_timeout);
 	if (processor_count == 1) {
-		poll_timer_add (instance->totemudpu_poll_handle,
-			instance->totem_config->downcheck_timeout,
+		qb_loop_timer_add (instance->totemudpu_poll_handle,
+			QB_LOOP_MED,
+			instance->totem_config->downcheck_timeout*QB_TIME_NS_IN_MSEC,
 			(void *)instance,
 			timer_function_netif_check_timeout,
 			&instance->timer_netif_check_timeout);
@@ -1579,12 +919,9 @@ extern int totemudpu_iface_check (void *udpu_context)
 extern void totemudpu_net_mtu_adjust (void *udpu_context, struct totem_config *totem_config)
 {
 #define UDPIP_HEADER_SIZE (20 + 8) /* 20 bytes for ip 8 bytes for udp */
-	if (totem_config->secauth == 1) {
-		totem_config->net_mtu -= sizeof (struct security_header) +
-			UDPIP_HEADER_SIZE;
-	} else {
-		totem_config->net_mtu -= UDPIP_HEADER_SIZE;
-	}
+	totem_config->net_mtu -= crypto_sec_header_size(totem_config->crypto_cipher_type,
+							totem_config->crypto_hash_type) +
+				 UDPIP_HEADER_SIZE;
 }
 
 const char *totemudpu_iface_print (void *udpu_context)  {
@@ -1641,12 +978,19 @@ extern int totemudpu_recv_mcast_empty (
 	msg_recv.msg_namelen = sizeof (struct sockaddr_storage);
 	msg_recv.msg_iov = &instance->totemudpu_iov_recv;
 	msg_recv.msg_iovlen = 1;
-#if !defined(COROSYNC_SOLARIS)
+#ifdef HAVE_MSGHDR_CONTROL
 	msg_recv.msg_control = 0;
+#endif
+#ifdef HAVE_MSGHDR_CONTROLLEN
 	msg_recv.msg_controllen = 0;
+#endif
+#ifdef HAVE_MSGHDR_FLAGS
 	msg_recv.msg_flags = 0;
-#else
+#endif
+#ifdef HAVE_MSGHDR_ACCRIGHTS
 	msg_recv.msg_accrights = NULL;
+#endif
+#ifdef HAVE_MSGHDR_ACCRIGHTSLEN
 	msg_recv.msg_accrightslen = 0;
 #endif
 
@@ -1746,8 +1090,48 @@ int totemudpu_member_remove (
 	void *udpu_context,
 	const struct totem_ip_address *token_target)
 {
+	int found = 0;
+	struct list_head *list;
+	struct totemudpu_member *member;
+
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
 
+	/*
+	 * Find the member to remove and close its socket
+	 */
+	for (list = instance->member_list.next;
+		list != &instance->member_list;
+		list = list->next) {
+
+		member = list_entry (list,
+			struct totemudpu_member,
+			list);
+
+		if (totemip_compare (token_target, &member->member)==0) {
+			log_printf(LOGSYS_LEVEL_NOTICE,
+				"removing UDPU member {%s}",
+				totemip_print(&member->member));
+
+			if (member->fd > 0) {
+				log_printf(LOGSYS_LEVEL_DEBUG,
+					"Closing socket to: {%s}",
+					totemip_print(&member->member));
+				qb_loop_poll_del (instance->totemudpu_poll_handle,
+					member->fd);
+				close (member->fd);
+			}
+			found = 1;
+			break;
+		}
+	}
+
+	/*
+	 * Delete the member from the list
+	 */
+	if (found) {
+		list_del (list);
+	}
+
 	instance = NULL;
 	return (0);
 }
