@@ -1,9 +1,10 @@
 /*
- * Copyright (c) 2009 Red Hat, Inc.
+ * Copyright (c) 2009-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
- * Author: Christine Caulfield (ccaulfie@redhat.com)
+ * Authors: Christine Caulfield (ccaulfie@redhat.com)
+ *          Fabio M. Di Nitto   (fdinitto@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -34,99 +35,117 @@
 #ifndef IPC_VOTEQUORUM_H_DEFINED
 #define IPC_VOTEQUORUM_H_DEFINED
 
-#include <corosync/corotypes.h>
 #include <corosync/mar_gen.h>
-#define VOTEQUORUM_MAX_QDISK_NAME_LEN 255
+#define VOTEQUORUM_QDEVICE_NODEID              0
+#define VOTEQUORUM_QDEVICE_MAX_NAME_LEN      255
+#define VOTEQUORUM_QDEVICE_DEFAULT_TIMEOUT 10000
 
 enum req_votequorum_types {
 	MESSAGE_REQ_VOTEQUORUM_GETINFO = 0,
 	MESSAGE_REQ_VOTEQUORUM_SETEXPECTED,
 	MESSAGE_REQ_VOTEQUORUM_SETVOTES,
-	MESSAGE_REQ_VOTEQUORUM_QDISK_REGISTER,
-	MESSAGE_REQ_VOTEQUORUM_QDISK_UNREGISTER,
-	MESSAGE_REQ_VOTEQUORUM_QDISK_POLL,
-	MESSAGE_REQ_VOTEQUORUM_QDISK_GETINFO,
-	MESSAGE_REQ_VOTEQUORUM_SETSTATE,
-	MESSAGE_REQ_VOTEQUORUM_LEAVING,
 	MESSAGE_REQ_VOTEQUORUM_TRACKSTART,
-	MESSAGE_REQ_VOTEQUORUM_TRACKSTOP
+	MESSAGE_REQ_VOTEQUORUM_TRACKSTOP,
+	MESSAGE_REQ_VOTEQUORUM_QDEVICE_REGISTER,
+	MESSAGE_REQ_VOTEQUORUM_QDEVICE_UNREGISTER,
+	MESSAGE_REQ_VOTEQUORUM_QDEVICE_UPDATE,
+	MESSAGE_REQ_VOTEQUORUM_QDEVICE_POLL,
+	MESSAGE_REQ_VOTEQUORUM_QDEVICE_MASTER_WINS
 };
 
 enum res_votequorum_types {
 	MESSAGE_RES_VOTEQUORUM_STATUS = 0,
 	MESSAGE_RES_VOTEQUORUM_GETINFO,
-	MESSAGE_RES_VOTEQUORUM_QDISK_GETINFO,
 	MESSAGE_RES_VOTEQUORUM_TRACKSTART,
 	MESSAGE_RES_VOTEQUORUM_NOTIFICATION,
 	MESSAGE_RES_VOTEQUORUM_EXPECTEDVOTES_NOTIFICATION
 };
 
+struct req_lib_votequorum_qdevice_register {
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
+	char name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
+};
+
+struct req_lib_votequorum_qdevice_unregister {
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
+	char name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
+};
+
+struct req_lib_votequorum_qdevice_update {
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
+	char oldname[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
+	char newname[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
+};
+
+struct req_lib_votequorum_qdevice_poll {
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
+	char name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
+	int cast_vote;
+};
+
+struct req_lib_votequorum_qdevice_master_wins {
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
+	char name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
+	unsigned int allow;
+};
+
 struct req_lib_votequorum_setvotes {
-        coroipc_request_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
 	unsigned int votes;
 	int nodeid;
 };
 
-struct req_lib_votequorum_qdisk_register {
-        coroipc_request_header_t header __attribute__((aligned(8)));
-	unsigned int votes;
-	char name[VOTEQUORUM_MAX_QDISK_NAME_LEN];
-};
-
-struct req_lib_votequorum_qdisk_poll {
-        coroipc_request_header_t header __attribute__((aligned(8)));
-	int state;
-};
-
 struct req_lib_votequorum_setexpected {
-        coroipc_request_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
 	unsigned int expected_votes;
 };
 
 struct req_lib_votequorum_trackstart {
-        coroipc_request_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
 	uint64_t context;
 	unsigned int track_flags;
 };
 
 struct req_lib_votequorum_general {
-        coroipc_request_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
 };
 
-#define VOTEQUORUM_REASON_KILL_REJECTED    1
-#define VOTEQUORUM_REASON_KILL_APPLICATION 2
-#define VOTEQUORUM_REASON_KILL_REJOIN      3
-
 struct req_lib_votequorum_getinfo {
-        coroipc_request_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_request_header header __attribute__((aligned(8)));
 	int nodeid;
 };
 
 struct res_lib_votequorum_status {
-        coroipc_response_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_response_header header __attribute__((aligned(8)));
 };
 
-#define VOTEQUORUM_INFO_FLAG_HASSTATE   1
-#define VOTEQUORUM_INFO_FLAG_DISALLOWED 2
-#define VOTEQUORUM_INFO_FLAG_TWONODE    4
-#define VOTEQUORUM_INFO_FLAG_QUORATE    8
+#define VOTEQUORUM_INFO_TWONODE                 1
+#define VOTEQUORUM_INFO_QUORATE                 2
+#define VOTEQUORUM_INFO_WAIT_FOR_ALL	        4
+#define VOTEQUORUM_INFO_LAST_MAN_STANDING       8
+#define VOTEQUORUM_INFO_AUTO_TIE_BREAKER       16
+#define VOTEQUORUM_INFO_ALLOW_DOWNSCALE        32
+#define VOTEQUORUM_INFO_QDEVICE_REGISTERED     64
+#define VOTEQUORUM_INFO_QDEVICE_ALIVE         128
+#define VOTEQUORUM_INFO_QDEVICE_CAST_VOTE     256
+#define VOTEQUORUM_INFO_QDEVICE_MASTER_WINS   512
+
+#define VOTEQUORUM_NODESTATE_MEMBER     1
+#define VOTEQUORUM_NODESTATE_DEAD       2
+#define VOTEQUORUM_NODESTATE_LEAVING    3
 
 struct res_lib_votequorum_getinfo {
-        coroipc_response_header_t header __attribute__((aligned(8)));
-	int nodeid;
+	struct qb_ipc_response_header header __attribute__((aligned(8)));
+	unsigned int nodeid;
+	unsigned int state;
 	unsigned int votes;
 	unsigned int expected_votes;
 	unsigned int highest_expected;
 	unsigned int total_votes;
 	unsigned int quorum;
 	unsigned int flags;
-};
-
-struct res_lib_votequorum_qdisk_getinfo {
-        coroipc_response_header_t header __attribute__((aligned(8)));
-	unsigned int votes;
-	unsigned int state;
-	char name[VOTEQUORUM_MAX_QDISK_NAME_LEN];
+	unsigned int qdevice_votes;
+	char qdevice_name[VOTEQUORUM_QDEVICE_MAX_NAME_LEN];
 };
 
 struct votequorum_node {
@@ -135,7 +154,7 @@ struct votequorum_node {
 };
 
 struct res_lib_votequorum_notification {
-	coroipc_response_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_response_header header __attribute__((aligned(8)));
 	mar_uint32_t quorate __attribute__((aligned(8)));
 	mar_uint64_t context __attribute__((aligned(8)));
 	mar_uint32_t node_list_entries __attribute__((aligned(8)));
@@ -143,7 +162,7 @@ struct res_lib_votequorum_notification {
 };
 
 struct res_lib_votequorum_expectedvotes_notification {
-	coroipc_response_header_t header __attribute__((aligned(8)));
+	struct qb_ipc_response_header header __attribute__((aligned(8)));
 	mar_uint64_t context __attribute__((aligned(8)));
 	mar_uint32_t expected_votes __attribute__((aligned(8)));
 };
