@@ -1,7 +1,7 @@
 /*
  * vi: set autoindent tabstop=4 shiftwidth=4 :
  *
- * Copyright (c) 2006-2009 Red Hat, Inc.
+ * Copyright (c) 2006-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -41,26 +41,35 @@
 #include <config.h>
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/mman.h>
+#include <sys/uio.h>
 #include <errno.h>
+#include <limits.h>
+
+#include <qb/qbdefs.h>
+#include <qb/qbipcc.h>
+#include <qb/qblog.h>
 
-#include <corosync/corotypes.h>
-#include <corosync/coroipc_types.h>
-#include <corosync/coroipcc.h>
-#include <corosync/corodefs.h>
 #include <corosync/hdb.h>
 #include <corosync/list.h>
-
+#include <corosync/corotypes.h>
+#include <corosync/corodefs.h>
 #include <corosync/cpg.h>
 #include <corosync/ipc_cpg.h>
 
 #include "util.h"
 
+#ifndef MAP_ANONYMOUS
+#define MAP_ANONYMOUS MAP_ANON
+#endif
+
 struct cpg_inst {
-	hdb_handle_t handle;
+	qb_ipcc_connection_t *c;
 	int finalize;
 	void *context;
 	union {
@@ -69,12 +78,13 @@ struct cpg_inst {
 	};
 	struct list_head iteration_list_head;
 };
+static void cpg_inst_free (void *inst);
 
-DECLARE_HDB_DATABASE(cpg_handle_t_db,NULL);
+DECLARE_HDB_DATABASE(cpg_handle_t_db, cpg_inst_free);
 
 struct cpg_iteration_instance_t {
 	cpg_iteration_handle_t cpg_iteration_handle;
-	hdb_handle_t conn_handle;
+	qb_ipcc_connection_t *conn;
 	hdb_handle_t executive_iteration_handle;
 	struct list_head list;
 };
@@ -86,12 +96,30 @@ DECLARE_HDB_DATABASE(cpg_iteration_handle_t_db,NULL);
  * Internal (not visible by API) functions
  */
 
+static cs_error_t
+coroipcc_msg_send_reply_receive (
+	qb_ipcc_connection_t *c,
+	const struct iovec *iov,
+	unsigned int iov_len,
+	void *res_msg,
+	size_t res_len)
+{
+	return qb_to_cs_error(qb_ipcc_sendv_recv(c, iov, iov_len, res_msg, res_len,
+				CS_IPC_TIMEOUT_MS));
+}
+
 static void cpg_iteration_instance_finalize (struct cpg_iteration_instance_t *cpg_iteration_instance)
 {
 	list_del (&cpg_iteration_instance->list);
 	hdb_handle_destroy (&cpg_iteration_handle_t_db, cpg_iteration_instance->cpg_iteration_handle);
 }
 
+static void cpg_inst_free (void *inst)
+{
+	struct cpg_inst *cpg_inst = (struct cpg_inst *)inst;
+	qb_ipcc_disconnect(cpg_inst->c);
+}
+
 static void cpg_inst_finalize (struct cpg_inst *cpg_inst, hdb_handle_t handle)
 {
 	struct list_head *iter, *iter_next;
@@ -143,7 +171,7 @@ cs_error_t cpg_model_initialize (
 	struct cpg_inst *cpg_inst;
 
 	if (model != CPG_MODEL_V1) {
-		error = CPG_ERR_INVALID_PARAM;
+		error = CS_ERR_INVALID_PARAM;
 		goto error_no_destroy;
 	}
 
@@ -157,14 +185,9 @@ cs_error_t cpg_model_initialize (
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (
-		COROSYNC_SOCKET_NAME,
-		CPG_SERVICE,
-		IPC_REQUEST_SIZE,
-		IPC_RESPONSE_SIZE,
-		IPC_DISPATCH_SIZE,
-		&cpg_inst->handle);
-	if (error != CS_OK) {
+	cpg_inst->c = qb_ipcc_connect ("cpg", IPC_REQUEST_SIZE);
+	if (cpg_inst->c == NULL) {
+		error = qb_to_cs_error(-errno);
 		goto error_put_destroy;
 	}
 
@@ -178,10 +201,6 @@ cs_error_t cpg_model_initialize (
 				goto error_destroy;
 			}
 			break;
-		default:
-			error = CS_ERR_LIBRARY;
-			goto error_destroy;
-			break;
 		}
 	}
 
@@ -221,7 +240,7 @@ cs_error_t cpg_finalize (
 	 */
 	if (cpg_inst->finalize) {
 		hdb_handle_put (&cpg_handle_t_db, handle);
-		return (CPG_ERR_BAD_HANDLE);
+		return (CS_ERR_BAD_HANDLE);
 	}
 
 	cpg_inst->finalize = 1;
@@ -235,14 +254,12 @@ cs_error_t cpg_finalize (
 	iov.iov_base = (void *)&req_lib_cpg_finalize;
 	iov.iov_len = sizeof (struct req_lib_cpg_finalize);
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->handle,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->c,
 		&iov,
 		1,
 		&res_lib_cpg_finalize,
 		sizeof (struct res_lib_cpg_finalize));
 
-	coroipcc_service_disconnect (cpg_inst->handle);
-
 	cpg_inst_finalize (cpg_inst, handle);
 	hdb_handle_put (&cpg_handle_t_db, handle);
 
@@ -261,7 +278,7 @@ cs_error_t cpg_fd_get (
 		return (error);
 	}
 
-	error = coroipcc_fd_get (cpg_inst->handle, fd);
+	error = qb_to_cs_error (qb_ipcc_fd_get (cpg_inst->c, fd));
 
 	hdb_handle_put (&cpg_handle_t_db, handle);
 
@@ -318,7 +335,7 @@ cs_error_t cpg_dispatch (
 	struct res_lib_cpg_deliver_callback *res_cpg_deliver_callback;
 	struct res_lib_cpg_totem_confchg_callback *res_cpg_totem_confchg_callback;
 	struct cpg_inst cpg_inst_copy;
-	coroipc_response_header_t *dispatch_data;
+	struct qb_ipc_response_header *dispatch_data;
 	struct cpg_address member_list[CPG_MEMBERS_MAX];
 	struct cpg_address left_list[CPG_MEMBERS_MAX];
 	struct cpg_address joined_list[CPG_MEMBERS_MAX];
@@ -328,6 +345,8 @@ cs_error_t cpg_dispatch (
 	unsigned int i;
 	struct cpg_ring_id ring_id;
 	uint32_t totem_member_list[CPG_MEMBERS_MAX];
+	int32_t errno_res;
+	char dispatch_buf[IPC_DISPATCH_SIZE];
 
 	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
@@ -335,25 +354,34 @@ cs_error_t cpg_dispatch (
 	}
 
 	/*
-	 * Timeout instantly for CS_DISPATCH_ONE or CS_DISPATCH_ALL and
-	 * wait indefinately for CS_DISPATCH_BLOCKING
+	 * Timeout instantly for CS_DISPATCH_ONE_NONBLOCKING or CS_DISPATCH_ALL and
+	 * wait indefinately for CS_DISPATCH_ONE or CS_DISPATCH_BLOCKING
 	 */
-	if (dispatch_types == CPG_DISPATCH_ALL) {
+	if (dispatch_types == CS_DISPATCH_ALL || dispatch_types == CS_DISPATCH_ONE_NONBLOCKING) {
 		timeout = 0;
 	}
 
+	dispatch_data = (struct qb_ipc_response_header *)dispatch_buf;
 	do {
-		error = coroipcc_dispatch_get (
-			cpg_inst->handle,
-			(void **)&dispatch_data,
+		errno_res = qb_ipcc_event_recv (
+			cpg_inst->c,
+			dispatch_buf,
+			IPC_DISPATCH_SIZE,
 			timeout);
+		error = qb_to_cs_error (errno_res);
 		if (error == CS_ERR_BAD_HANDLE) {
 			error = CS_OK;
 			goto error_put;
 		}
 		if (error == CS_ERR_TRY_AGAIN) {
+			if (dispatch_types == CS_DISPATCH_ONE_NONBLOCKING) {
+				/*
+				 * Don't mask error
+				 */
+				goto error_put;
+			}
 			error = CS_OK;
-			if (dispatch_types == CPG_DISPATCH_ALL) {
+			if (dispatch_types == CS_DISPATCH_ALL) {
 				break; /* exit do while cont is 1 loop */
 			} else {
 				continue; /* next poll */
@@ -369,7 +397,6 @@ cs_error_t cpg_dispatch (
 		 * operate at the same time that cpgFinalize has been called.
 		 */
 		memcpy (&cpg_inst_copy, cpg_inst, sizeof (struct cpg_inst));
-
 		switch (cpg_inst_copy.model_data.model) {
 		case CPG_MODEL_V1:
 			/*
@@ -451,24 +478,26 @@ cs_error_t cpg_dispatch (
 					totem_member_list);
 				break;
 			default:
-				error = coroipcc_dispatch_put (cpg_inst->handle);
-				if (error == CS_OK) {
-					error = CS_ERR_LIBRARY;
-				}
+				error = CS_ERR_LIBRARY;
 				goto error_put;
 				break;
 			} /* - switch (dispatch_data->id) */
 			break; /* case CPG_MODEL_V1 */
 		} /* - switch (cpg_inst_copy.model_data.model) */
-		error = coroipcc_dispatch_put (cpg_inst->handle);
-		if (error != CS_OK) {
+
+		if (cpg_inst_copy.finalize || cpg_inst->finalize) {
+			/*
+			 * If the finalize has been called then get out of the dispatch.
+			 */
+			cpg_inst->finalize = 1;
+			error = CS_ERR_BAD_HANDLE;
 			goto error_put;
 		}
 
 		/*
 		 * Determine if more messages should be processed
 		 */
-		if (dispatch_types == CS_DISPATCH_ONE) {
+		if (dispatch_types == CS_DISPATCH_ONE || dispatch_types == CS_DISPATCH_ONE_NONBLOCKING) {
 			cont = 0;
 		}
 	} while (cont);
@@ -486,7 +515,7 @@ cs_error_t cpg_join (
 	struct cpg_inst *cpg_inst;
 	struct iovec iov[2];
 	struct req_lib_cpg_join req_lib_cpg_join;
-	struct res_lib_cpg_join res_lib_cpg_join;
+	struct res_lib_cpg_join response;
 
 	if (group->length > CPG_MAX_NAME_LENGTH) {
 		return (CS_ERR_NAME_TOO_LONG);
@@ -516,15 +545,15 @@ cs_error_t cpg_join (
 	iov[0].iov_len = sizeof (struct req_lib_cpg_join);
 
 	do {
-		error = coroipcc_msg_send_reply_receive (cpg_inst->handle, iov, 1,
-			&res_lib_cpg_join, sizeof (struct res_lib_cpg_join));
+		error = coroipcc_msg_send_reply_receive (cpg_inst->c, iov, 1,
+			&response, sizeof (struct res_lib_cpg_join));
 
 		if (error != CS_OK) {
 			goto error_exit;
 		}
-	} while (res_lib_cpg_join.header.error == CPG_ERR_BUSY);
+	} while (response.header.error == CS_ERR_BUSY);
 
-	error = res_lib_cpg_join.header.error;
+	error = response.header.error;
 
 error_exit:
 	hdb_handle_put (&cpg_handle_t_db, handle);
@@ -561,13 +590,13 @@ cs_error_t cpg_leave (
 	iov[0].iov_len = sizeof (struct req_lib_cpg_leave);
 
 	do {
-		error = coroipcc_msg_send_reply_receive (cpg_inst->handle, iov, 1,
+		error = coroipcc_msg_send_reply_receive (cpg_inst->c, iov, 1,
 			&res_lib_cpg_leave, sizeof (struct res_lib_cpg_leave));
 
 		if (error != CS_OK) {
 			goto error_exit;
 		}
-	} while (res_lib_cpg_leave.header.error == CPG_ERR_BUSY);
+	} while (res_lib_cpg_leave.header.error == CS_ERR_BUSY);
 
 	error = res_lib_cpg_leave.header.error;
 
@@ -608,20 +637,18 @@ cs_error_t cpg_membership_get (
 	req_lib_cpg_membership_get.header.size = sizeof (struct req_lib_cpg_membership_get);
 	req_lib_cpg_membership_get.header.id = MESSAGE_REQ_CPG_MEMBERSHIP;
 
-	memcpy (&req_lib_cpg_membership_get.group_name, group_name,
-		sizeof (struct cpg_name));
+	marshall_to_mar_cpg_name_t (&req_lib_cpg_membership_get.group_name,
+		group_name);
 
 	iov.iov_base = (void *)&req_lib_cpg_membership_get;
-	iov.iov_len = sizeof (coroipc_request_header_t);
+	iov.iov_len = sizeof (struct req_lib_cpg_membership_get);
 
-	do {
-		error = coroipcc_msg_send_reply_receive (cpg_inst->handle, &iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->c, &iov, 1,
 			&res_lib_cpg_membership_get, sizeof (res_lib_cpg_membership_get));
 
- 		if (error != CS_OK) {
- 			goto error_exit;
-		}
-	} while (res_lib_cpg_membership_get.header.error == CPG_ERR_BUSY);
+	if (error != CS_OK) {
+		goto error_exit;
+	}
 
 	error = res_lib_cpg_membership_get.header.error;
 
@@ -657,13 +684,13 @@ cs_error_t cpg_local_get (
 		return (error);
 	}
 
-	req_lib_cpg_local_get.header.size = sizeof (coroipc_request_header_t);
+	req_lib_cpg_local_get.header.size = sizeof (struct qb_ipc_request_header);
 	req_lib_cpg_local_get.header.id = MESSAGE_REQ_CPG_LOCAL_GET;
 
 	iov.iov_base = (void *)&req_lib_cpg_local_get;
 	iov.iov_len = sizeof (struct req_lib_cpg_local_get);
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->handle, &iov, 1,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->c, &iov, 1,
 		&res_lib_cpg_local_get, sizeof (res_lib_cpg_local_get));
 
 	if (error != CS_OK) {
@@ -691,19 +718,103 @@ cs_error_t cpg_flow_control_state_get (
 	if (error != CS_OK) {
 		return (error);
 	}
-
-	error = coroipcc_dispatch_flow_control_get (cpg_inst->handle, (unsigned int *)flow_control_state);
+	*flow_control_state = CPG_FLOW_CONTROL_DISABLED;
+	error = CS_OK;
 
 	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
 }
 
+static int
+memory_map (char *path, const char *file, void **buf, size_t bytes)
+{
+	int32_t fd;
+	void *addr_orig;
+	void *addr;
+	int32_t res;
+	char *buffer;
+	int32_t i;
+	size_t written;
+	size_t page_size; 
+
+	snprintf (path, PATH_MAX, "/dev/shm/%s", file);
+
+	fd = mkstemp (path);
+	if (fd == -1) {
+		snprintf (path, PATH_MAX, LOCALSTATEDIR "/run/%s", file);
+		fd = mkstemp (path);
+		if (fd == -1) {
+			return (-1);
+		}
+	}
+
+	res = ftruncate (fd, bytes);
+	if (res == -1) {
+		goto error_close_unlink;
+	}
+	page_size = (size_t)sysconf(_SC_PAGESIZE);
+	buffer = malloc (page_size);
+	if (buffer == NULL) {
+		goto error_close_unlink;
+	}
+	memset (buffer, 0, page_size);
+	for (i = 0; i < (bytes / page_size); i++) {
+retry_write:
+		written = write (fd, buffer, page_size);
+		if (written == -1 && errno == EINTR) {
+			goto retry_write;
+		}
+		if (written != page_size) {
+			free (buffer);
+			goto error_close_unlink;
+		}
+	}
+	free (buffer);
+
+	addr_orig = mmap (NULL, bytes, PROT_NONE,
+		MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+
+	if (addr_orig == MAP_FAILED) {
+		goto error_close_unlink;
+	}
+
+	addr = mmap (addr_orig, bytes, PROT_READ | PROT_WRITE,
+		MAP_FIXED | MAP_SHARED, fd, 0);
+
+	if (addr != addr_orig) {
+		goto error_close_unlink;
+	}
+#ifdef MADV_NOSYNC
+	madvise(addr_orig, bytes, MADV_NOSYNC);
+#endif
+
+	res = close (fd);
+	if (res) {
+		return (-1);
+	}
+	*buf = addr_orig;
+
+	return 0;
+
+error_close_unlink:
+	close (fd);
+	unlink(path);
+	return -1;
+}
+
 cs_error_t cpg_zcb_alloc (
 	cpg_handle_t handle,
 	size_t size,
 	void **buffer)
 {
+	void *buf = NULL;
+	char path[PATH_MAX];
+	mar_req_coroipcc_zc_alloc_t req_coroipcc_zc_alloc;
+	struct qb_ipc_response_header res_coroipcs_zc_alloc;
+	size_t map_size;
+	struct iovec iovec;
+	struct coroipcs_zc_header *hdr;
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
 
@@ -712,10 +823,27 @@ cs_error_t cpg_zcb_alloc (
 		return (error);
 	}
 
-	error = coroipcc_zcb_alloc (cpg_inst->handle,
-		buffer,
-		size,
-		sizeof (struct req_lib_cpg_mcast));
+	map_size = size + sizeof (struct req_lib_cpg_mcast) + sizeof (struct coroipcs_zc_header);
+	assert(memory_map (path, "corosync_zerocopy-XXXXXX", &buf, map_size) != -1);
+
+	req_coroipcc_zc_alloc.header.size = sizeof (mar_req_coroipcc_zc_alloc_t);
+	req_coroipcc_zc_alloc.header.id = MESSAGE_REQ_CPG_ZC_ALLOC;
+	req_coroipcc_zc_alloc.map_size = map_size;
+	strcpy (req_coroipcc_zc_alloc.path_to_file, path);
+
+	iovec.iov_base = (void *)&req_coroipcc_zc_alloc;
+	iovec.iov_len = sizeof (mar_req_coroipcc_zc_alloc_t);
+
+	error = coroipcc_msg_send_reply_receive (
+		cpg_inst->c,
+		&iovec,
+		1,
+		&res_coroipcs_zc_alloc,
+		sizeof (struct qb_ipc_response_header));
+
+	hdr = (struct coroipcs_zc_header *)buf;
+	hdr->map_size = map_size;
+	*buffer = ((char *)buf) + sizeof (struct coroipcs_zc_header);
 
 	hdb_handle_put (&cpg_handle_t_db, handle);
 	*buffer = ((char *)*buffer) + sizeof (struct req_lib_cpg_mcast);
@@ -729,13 +857,32 @@ cs_error_t cpg_zcb_free (
 {
 	cs_error_t error;
 	struct cpg_inst *cpg_inst;
+	mar_req_coroipcc_zc_free_t req_coroipcc_zc_free;
+	struct qb_ipc_response_header res_coroipcs_zc_free;
+	struct iovec iovec;
+	struct coroipcs_zc_header *header = (struct coroipcs_zc_header *)((char *)buffer - sizeof (struct coroipcs_zc_header));
 
 	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
 
-	coroipcc_zcb_free (cpg_inst->handle, ((char *)buffer) - sizeof (struct req_lib_cpg_mcast));
+	req_coroipcc_zc_free.header.size = sizeof (mar_req_coroipcc_zc_free_t);
+	req_coroipcc_zc_free.header.id = MESSAGE_REQ_CPG_ZC_FREE;
+	req_coroipcc_zc_free.map_size = header->map_size;
+	req_coroipcc_zc_free.server_address = header->server_address;
+
+	iovec.iov_base = (void *)&req_coroipcc_zc_free;
+	iovec.iov_len = sizeof (mar_req_coroipcc_zc_free_t);
+
+	error = coroipcc_msg_send_reply_receive (
+		cpg_inst->c,
+		&iovec,
+		1,
+		&res_coroipcs_zc_free,
+		sizeof (struct qb_ipc_response_header));
+
+	munmap ((void *)header, header->map_size);
 
 	hdb_handle_put (&cpg_handle_t_db, handle);
 
@@ -752,12 +899,14 @@ cs_error_t cpg_zcb_mcast_joined (
 	struct cpg_inst *cpg_inst;
 	struct req_lib_cpg_mcast *req_lib_cpg_mcast;
 	struct res_lib_cpg_mcast res_lib_cpg_mcast;
+	mar_req_coroipcc_zc_execute_t req_coroipcc_zc_execute;
+	struct coroipcs_zc_header *hdr;
+	struct iovec iovec;
 
 	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
 	if (error != CS_OK) {
 		return (error);
 	}
-
 	req_lib_cpg_mcast = (struct req_lib_cpg_mcast *)(((char *)msg) - sizeof (struct req_lib_cpg_mcast));
 	req_lib_cpg_mcast->header.size = sizeof (struct req_lib_cpg_mcast) +
 		msg_len;
@@ -766,11 +915,21 @@ cs_error_t cpg_zcb_mcast_joined (
 	req_lib_cpg_mcast->guarantee = guarantee;
 	req_lib_cpg_mcast->msglen = msg_len;
 
-	error = coroipcc_zcb_msg_send_reply_receive (
-		cpg_inst->handle,
-		req_lib_cpg_mcast,
+	hdr = (struct coroipcs_zc_header *)(((char *)req_lib_cpg_mcast) - sizeof (struct coroipcs_zc_header));
+
+	req_coroipcc_zc_execute.header.size = sizeof (mar_req_coroipcc_zc_execute_t);
+	req_coroipcc_zc_execute.header.id = MESSAGE_REQ_CPG_ZC_EXECUTE;
+	req_coroipcc_zc_execute.server_address = hdr->server_address;
+
+	iovec.iov_base = (void *)&req_coroipcc_zc_execute;
+	iovec.iov_len = sizeof (mar_req_coroipcc_zc_execute_t);
+
+	error = coroipcc_msg_send_reply_receive (
+		cpg_inst->c,
+		&iovec,
+		1,
 		&res_lib_cpg_mcast,
-		sizeof (res_lib_cpg_mcast));
+		sizeof(res_lib_cpg_mcast));
 
 	if (error != CS_OK) {
 		goto error_exit;
@@ -795,7 +954,6 @@ cs_error_t cpg_mcast_joined (
 	struct cpg_inst *cpg_inst;
 	struct iovec iov[64];
 	struct req_lib_cpg_mcast req_lib_cpg_mcast;
-	struct res_lib_cpg_mcast res_lib_cpg_mcast;
 	size_t msg_len = 0;
 
 	error = hdb_error_to_cs (hdb_handle_get (&cpg_handle_t_db, handle, (void *)&cpg_inst));
@@ -818,16 +976,10 @@ cs_error_t cpg_mcast_joined (
 	iov[0].iov_len = sizeof (struct req_lib_cpg_mcast);
 	memcpy (&iov[1], iovec, iov_len * sizeof (struct iovec));
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->handle, iov,
-		iov_len + 1, &res_lib_cpg_mcast, sizeof (res_lib_cpg_mcast));
+	qb_ipcc_fc_enable_max_set(cpg_inst->c,  2);
+	error = qb_to_cs_error(qb_ipcc_sendv(cpg_inst->c, iov, iov_len + 1));
+	qb_ipcc_fc_enable_max_set(cpg_inst->c,  1);
 
-	if (error != CS_OK) {
-		goto error_exit;
-	}
-
-	error = res_lib_cpg_mcast.header.error;
-
-error_exit:
 	hdb_handle_put (&cpg_handle_t_db, handle);
 
 	return (error);
@@ -881,7 +1033,7 @@ cs_error_t cpg_iteration_initialize(
 		goto error_destroy;
 	}
 
-	cpg_iteration_instance->conn_handle = cpg_inst->handle;
+	cpg_iteration_instance->conn = cpg_inst->c;
 
 	list_init (&cpg_iteration_instance->list);
 
@@ -895,7 +1047,7 @@ cs_error_t cpg_iteration_initialize(
 	iov.iov_base = (void *)&req_lib_cpg_iterationinitialize;
 	iov.iov_len = sizeof (struct req_lib_cpg_iterationinitialize);
 
-	error = coroipcc_msg_send_reply_receive (cpg_inst->handle,
+	error = coroipcc_msg_send_reply_receive (cpg_inst->c,
 		&iov,
 		1,
 		&res_lib_cpg_iterationinitialize,
@@ -933,9 +1085,7 @@ cs_error_t cpg_iteration_next(
 	cs_error_t error;
 	struct cpg_iteration_instance_t *cpg_iteration_instance;
 	struct req_lib_cpg_iterationnext req_lib_cpg_iterationnext;
-	struct res_lib_cpg_iterationnext *res_lib_cpg_iterationnext;
-	struct iovec iov;
-	void *return_address;
+	struct res_lib_cpg_iterationnext res_lib_cpg_iterationnext;
 
 	if (description == NULL) {
 		return CS_ERR_INVALID_PARAM;
@@ -951,27 +1101,25 @@ cs_error_t cpg_iteration_next(
 	req_lib_cpg_iterationnext.header.id = MESSAGE_REQ_CPG_ITERATIONNEXT;
 	req_lib_cpg_iterationnext.iteration_handle = cpg_iteration_instance->executive_iteration_handle;
 
-	iov.iov_base = (void *)&req_lib_cpg_iterationnext;
-	iov.iov_len = sizeof (struct req_lib_cpg_iterationnext);
-
-	error = coroipcc_msg_send_reply_receive_in_buf_get (cpg_iteration_instance->conn_handle,
-		&iov,
-		1,
-		&return_address);
-	res_lib_cpg_iterationnext = return_address;
+	error = qb_to_cs_error (qb_ipcc_send (cpg_iteration_instance->conn,
+				&req_lib_cpg_iterationnext,
+				req_lib_cpg_iterationnext.header.size));
+	if (error != CS_OK) {
+		goto error_put;
+	}
 
+	error = qb_to_cs_error (qb_ipcc_recv (cpg_iteration_instance->conn,
+				&res_lib_cpg_iterationnext,
+				sizeof(struct res_lib_cpg_iterationnext), -1));
 	if (error != CS_OK) {
 		goto error_put;
 	}
 
 	marshall_from_mar_cpg_iteration_description_t(
 			description,
-			&res_lib_cpg_iterationnext->description);
+			&res_lib_cpg_iterationnext.description);
 
-	error = res_lib_cpg_iterationnext->header.error;
-
-	coroipcc_msg_send_reply_receive_in_buf_put(
-			cpg_iteration_instance->conn_handle);
+	error = res_lib_cpg_iterationnext.header.error;
 
 error_put:
 	hdb_handle_put (&cpg_iteration_handle_t_db, handle);
@@ -1002,7 +1150,7 @@ cs_error_t cpg_iteration_finalize (
 	iov.iov_base = (void *)&req_lib_cpg_iterationfinalize;
 	iov.iov_len = sizeof (struct req_lib_cpg_iterationfinalize);
 
-	error = coroipcc_msg_send_reply_receive (cpg_iteration_instance->conn_handle,
+	error = coroipcc_msg_send_reply_receive (cpg_iteration_instance->conn,
 		&iov,
 		1,
 		&res_lib_cpg_iterationfinalize,
